<?php

namespace Drupal\commerce_favorites\Access;

use Drupal\commerce_favorites\FavoritesSessionInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Access check for the favorites item details_form route.
 */
class FavoritesItemDetailsAccessCheck implements AccessInterface {

  /**
   * The favorites session.
   *
   * @var \Drupal\commerce_favorites\FavoritesSessionInterface
   */
  protected $favoritesSession;

  /**
   * Constructs a new FavoritesItemDetailsAccessCheck object.
   *
   * @param \Drupal\commerce_favorites\FavoritesSessionInterface $favorites_session
   *   The favorites session.
   */
  public function __construct(FavoritesSessionInterface $favorites_session) {
    $this->favoritesSession = $favorites_session;
  }

  /**
   * Checks access to the favorites item details form.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(RouteMatchInterface $route_match, AccountInterface $account) {
    if ($account->hasPermission('administer commerce_favorites')) {
      // Administrators can modify anyone's favorites list.
      $access = AccessResult::allowed()->cachePerPermissions();
    }
    else {
      // Users can modify their own favorites_lists.
      /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
      $favorites_item = $route_match->getParameter('commerce_favorites_item');
      $user = $favorites_item->getFavorites()->getOwner();

      if ($account->isAuthenticated()) {
        $access = AccessResult::allowedIf($user->id() === $account->id())
          ->addCacheableDependency($favorites_item)
          ->cachePerUser();
      }
      else {
        $access = AccessResult::allowedIf($this->favoritesSession->hasFavoritesId($favorites_item->getFavoritesId()))
          ->addCacheableDependency($favorites_item)
          ->addCacheContexts(['favorites']);
      }
    }

    return $access;
  }

}
