<?php

namespace Drupal\commerce_favorites\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\commerce_favorites\FavoritesProviderInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides the favorites pages.
 */
class FavoritesController implements ContainerInjectionInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The favorites provider.
   *
   * @var \Drupal\commerce_favorites\FavoritesProviderInterface
   */
  protected $favoritesProvider;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new FavoritesController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\commerce_favorites\FavoritesProviderInterface $favorites_provider
   *   The favorites provider.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager, FormBuilderInterface $form_builder, RouteMatchInterface $route_match, FavoritesProviderInterface $favorites_provider, LanguageManagerInterface $language_manager) {
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->formBuilder = $form_builder;
    $this->routeMatch = $route_match;
    $this->favoritesProvider = $favorites_provider;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('form_builder'),
      $container->get('current_route_match'),
      $container->get('commerce_favorites.favorites_provider'),
      $container->get('language_manager')
    );
  }

  /**
   * Builds the favorites page.
   *
   * If the customer doesn't have a favorites, or the favorites is empty,
   * the "empty page" will be shown. Otherwise, the customer will be redirected
   * to the default favorites.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   A render array, or a redirect response.
   */
  public function favoritesPage() {
    $favorites = $this->favoritesProvider->getFavorites($this->getDefaultFavoritesType());
    if (!$favorites || !$favorites->hasItems()) {
      return [
        '#theme' => 'commerce_favorites_empty_page',
        '#cache' => [
          'contexts' => ['user', 'session'],
        ],
      ];
    }
    // Authenticated users should always manage favorites_lists via the user form.
    $rel = $this->currentUser->isAuthenticated() ? 'user-form' : 'canonical';
    $url = $favorites->toUrl($rel, [
      'absolute' => TRUE,
      'language' => $this->languageManager->getCurrentLanguage(),
    ]);

    return new RedirectResponse($url->toString());
  }

  /**
   * Builds the user favorites page.
   *
   * If the customer doesn't have a favorites, or the favorites is empty,
   * the "empty page" will be shown. Otherwise, the customer will be redirected
   * to the default favorites.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   A render array, or a redirect response.
   */
  public function userPage() {
    $favorites = $this->favoritesProvider->getFavorites($this->getDefaultFavoritesType());
    if (!$favorites || !$favorites->hasItems()) {
      return [
        '#theme' => 'commerce_favorites_empty_page',
      ];
    }
    $url = $favorites->toUrl('user-form', ['absolute' => TRUE]);

    return new RedirectResponse($url->toString());
  }

  /**
   * Builds the user form.
   *
   * @return array
   *   The rendered form.
   */
  public function userForm() {
    $form_object = $this->getFormObject('user');
    $form_state = new FormState();

    return $this->formBuilder->buildForm($form_object, $form_state);
  }

  /**
   * Builds the share form.
   *
   * @return array
   *   The rendered form.
   */
  public function shareForm() {
    $form_object = $this->getFormObject('share');
    $form_state = new FormState();

    return $this->formBuilder->buildForm($form_object, $form_state);
  }

  /**
   * Gets the form object for the given operation.
   *
   * @param string $operation
   *   The operation.
   *
   * @return \Drupal\Core\Entity\EntityFormInterface
   *   The form object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   Thrown if no favorites with the code specified in the URL could be found.
   */
  protected function getFormObject($operation) {
    $code = $this->routeMatch->getRawParameter('code');
    /** @var \Drupal\commerce_favorites\FavoritesStorageInterface $favorites_storage */
    $favorites_storage = $this->entityTypeManager->getStorage('commerce_favorites');
    $favorites = $favorites_storage->loadByCode($code);
    if (!$favorites) {
      throw new NotFoundHttpException();
    }
    $form_object = $this->entityTypeManager->getFormObject('commerce_favorites', $operation);
    $form_object->setEntity($favorites);

    return $form_object;
  }

  /**
   * Gets the default favorites type.
   */
  protected function getDefaultFavoritesType() {
    return $this->configFactory->get('commerce_favorites.settings')->get('default_type');
  }

}
