<?php

namespace Drupal\commerce_favorites\Entity;

use Drupal\Component\Utility\Random;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Url;
use Drupal\profile\Entity\ProfileInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the favorites entity class.
 *
 * @ContentEntityType(
 *   id = "commerce_favorites",
 *   label = @Translation("Favorites"),
 *   label_collection = @Translation("Favorites Lists"),
 *   label_singular = @Translation("favorites"),
 *   label_plural = @Translation("favorites_lists"),
 *   label_count = @PluralTranslation(
 *     singular = "@count favorites",
 *     plural = "@count favorites_lists",
 *   ),
 *   bundle_label = @Translation("Favorites type"),
 *   handlers = {
 *     "event" = "Drupal\commerce_favorites\Event\FavoritesEvent",
 *     "storage" = "Drupal\commerce_favorites\FavoritesStorage",
 *     "access" = "Drupal\entity\UncacheableEntityAccessControlHandler",
 *     "query_access" = "Drupal\entity\QueryAccess\UncacheableQueryAccessHandler",
 *     "permission_provider" = "Drupal\entity\UncacheableEntityPermissionProvider",
 *     "list_builder" = "Drupal\commerce_favorites\FavoritesListBuilder",
 *     "views_data" = "Drupal\commerce\CommerceEntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\commerce_favorites\Form\FavoritesForm",
 *       "add" = "Drupal\commerce_favorites\Form\FavoritesForm",
 *       "edit" = "Drupal\commerce_favorites\Form\FavoritesForm",
 *       "duplicate" = "Drupal\commerce_favorites\Form\FavoritesForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "user" = "Drupal\commerce_favorites\Form\FavoritesUserForm",
 *       "share" = "Drupal\commerce_favorites\Form\FavoritesShareForm",
 *     },
 *     "local_task_provider" = {
 *       "default" = "Drupal\entity\Menu\DefaultEntityLocalTaskProvider",
 *     },
 *     "route_provider" = {
 *       "default" = "Drupal\commerce_favorites\FavoritesRouteProvider",
 *       "delete-multiple" = "Drupal\entity\Routing\DeleteMultipleRouteProvider",
 *     },
 *   },
 *   base_table = "commerce_favorites",
 *   admin_permission = "administer commerce_favorites",
 *   permission_granularity = "bundle",
 *   fieldable = TRUE,
 *   entity_keys = {
 *     "id" = "favorites_id",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "bundle" = "type",
 *     "uid" = "uid",
 *     "owner" = "uid",
 *   },
 *   links = {
 *     "add-page" = "/admin/commerce/favorites/add",
 *     "add-form" = "/admin/commerce/favorites/add/{commerce_favorites_type}",
 *     "edit-form" = "/admin/commerce/favorites/{commerce_favorites}/edit",
 *     "duplicate-form" = "/admin/commerce/favorites/{commerce_favorites}/duplicate",
 *     "delete-form" = "/admin/commerce/favorites/{commerce_favorites}/delete",
 *     "delete-multiple-form" = "/admin/commerce/favorites/delete",
 *     "collection" = "/admin/commerce/favorites",
 *     "user-form" = "/user/{user}/favorites/{code}",
 *     "share-form" = "/user/{user}/favorites/{code}/share",
 *   },
 *   bundle_entity_type = "commerce_favorites_type",
 *   field_ui_base_route = "entity.commerce_favorites_type.edit_form"
 * )
 */
class Favorites extends ContentEntityBase implements FavoritesInterface {

  use EntityChangedTrait;
  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function createDuplicate() {
    $duplicate = parent::createDuplicate();
    // Unique code cannot be transferred because their codes are unique.
    $duplicate->set('code', NULL);
    // We don't duplicate favorites items.
    $duplicate->set('favorites_items', []);

    return $duplicate;
  }

  /**
   * {@inheritdoc}
   */
  public function toUrl($rel = 'canonical', array $options = []) {
    // Can't declare "canonical" as a link template because it requires a
    // custom parameter, which breaks contribs that don't expect it.
    // StringFormatter assumes 'revision' is always a valid link template.
    if (in_array($rel, ['canonical', 'revision'])) {
      $route_name = 'entity.commerce_favorites.canonical';
      $route_parameters = [
        'code' => $this->getCode(),
      ];
      $options += [
        'entity_type' => 'commerce_favorites',
        'entity' => $this,
        // Display links by default based on the current language.
        'language' => $this->language(),
      ];
      return new Url($route_name, $route_parameters, $options);
    }
    else {
      return parent::toUrl($rel, $options);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    if (in_array($rel, ['user-form', 'share-form'])) {
      return [
        'user' => $this->getOwnerId(),
        'code' => $this->getCode(),
      ];
    }
    else {
      return parent::urlRouteParameters($rel);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCode() {
    return $this->get('code')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCode($code) {
    $this->set('code', $code);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getShippingProfile() {
    return $this->get('shipping_profile')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setShippingProfile(ProfileInterface $profile) {
    $this->set('shipping_profile', $profile);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getItems() {
    return $this->get('favorites_items')->referencedEntities();
  }

  /**
   * {@inheritdoc}
   */
  public function setItems(array $favorites_items) {
    $this->set('favorites_items', $favorites_items);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function hasItems() {
    return !$this->get('favorites_items')->isEmpty();
  }

  /**
   * {@inheritdoc}
   */
  public function addItem(FavoritesItemInterface $favorites_item) {
    if (!$this->hasItem($favorites_item)) {
      $this->get('favorites_items')->appendItem($favorites_item);
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function removeItem(FavoritesItemInterface $favorites_item) {
    $index = $this->getItemIndex($favorites_item);
    if ($index !== FALSE) {
      $this->get('favorites_items')->offsetUnset($index);
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function hasItem(FavoritesItemInterface $favorites_item) {
    return $this->getItemIndex($favorites_item) !== FALSE;
  }

  /**
   * Gets the index of the given favorites item.
   *
   * @param \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item
   *   The favorites item.
   *
   * @return int|bool
   *   The index of the given favorites item, or FALSE if not found.
   */
  protected function getItemIndex(FavoritesItemInterface $favorites_item) {
    $values = $this->get('favorites_items')->getValue();
    $favorites_item_ids = array_map(function ($value) {
      return $value['target_id'];
    }, $values);

    return array_search($favorites_item->id(), $favorites_item_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function isDefault() {
    return (bool) $this->get('is_default')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setDefault($default) {
    $this->set('is_default', (bool) $default);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublic() {
    return (bool) $this->get('is_public')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setPublic($public) {
    $this->set('is_public', (bool) $public);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getKeepPurchasedItems() {
    return (bool) $this->get('keep_purchased_items')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setKeepPurchasedItems($keep_purchased_items) {
    $this->set('keep_purchased_items', (bool) $keep_purchased_items);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    /** @var \Drupal\commerce_favorites\FavoritesStorageInterface $storage */
    parent::preSave($storage);

    if ($this->get('code')->isEmpty()) {
      /** @var \Drupal\commerce_favorites\FavoritesStorageInterface $storage */
      $storage = $this->entityTypeManager()->getStorage('commerce_favorites');
      $random = new Random();
      $code = $random->word(13);
      // Ensure code uniqueness. Collisions are rare, but possible.
      while ($storage->loadByCode($code)) {
        $code = $random->word(13);
      }
      $this->setCode($random->word(13));
    }
    // Mark the favorites as default if there's no other default.
    if ($this->getOwnerId() && !$this->isDefault()) {
      $favorites = $storage->loadDefaultByUser($this->getOwner(), $this->bundle());
      if (!$favorites || !$favorites->isDefault()) {
        $this->setDefault(TRUE);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    /** @var \Drupal\commerce_favorites\FavoritesStorageInterface $storage */
    parent::postSave($storage, $update);

    // Ensure there's a back-reference on each favorites item.
    foreach ($this->getItems() as $favorites_item) {
      if ($favorites_item->favorites_id->isEmpty()) {
        $favorites_item->favorites_id = $this->id();
        $favorites_item->save();
      }
    }

    if ($this->getOwnerId()) {
      $default = $this->isDefault();
      $original_default = $this->original ? $this->original->isDefault() : FALSE;
      if ($default && !$original_default) {
        $favorites_lists = $storage->loadMultipleByUser($this->getOwner(), $this->bundle());
        foreach ($favorites_lists as $favorites) {
          if ($favorites->id() != $this->id() && $favorites->isDefault()) {
            $favorites->setDefault(FALSE);
            $favorites->save();
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);

    // Delete the favorites items of a deleted favorites.
    $favorites_items = [];
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $entity */
    foreach ($entities as $entity) {
      foreach ($entity->getItems() as $favorites_item) {
        $favorites_items[$favorites_item->id()] = $favorites_item;
      }
    }
    /** @var \Drupal\commerce_favorites\FavoritesItemStorageInterface $favorites_item_storage */
    $favorites_item_storage = \Drupal::service('entity_type.manager')->getStorage('commerce_favorites_item');
    $favorites_item_storage->delete($favorites_items);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['code'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Code'))
      ->setDescription(t('The favorites code.'))
      ->setSetting('max_length', 25)
      ->addConstraint('UniqueField', []);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The favorites name.'))
      ->setRequired(TRUE)
      ->setDefaultValue('')
      ->setSetting('max_length', 255)
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid']->setLabel(t('Owner'))
      ->setDescription(t('The favorites owner.'))
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['shipping_profile'] = BaseFieldDefinition::create('entity_reference_revisions')
      ->setLabel(t('Shipping profile'))
      ->setDescription(t('Shipping profile'))
      ->setSetting('target_type', 'profile')
      ->setSetting('handler', 'default')
      ->setSetting('handler_settings', ['target_bundles' => ['customer']])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 0,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['favorites_items'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Favorites items'))
      ->setDescription(t('The favorites items.'))
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setSetting('target_type', 'commerce_favorites_item')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'type' => 'commerce_favorites_item_table',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['is_default'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Default'))
      ->setDescription(t('A boolean indicating whether the favorites is the default one.'));

    $fields['is_public'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Public'))
      ->setDescription(t('Whether the favorites is public.'))
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 19,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['keep_purchased_items'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Keep purchased items in the list'))
      ->setDescription(t('Whether items should remain in the favorites once purchased.'))
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time when the favorites was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time when the favorites was last edited.'));

    return $fields;
  }

}
