<?php

namespace Drupal\commerce_favorites\Entity;

use Drupal\commerce_favorites\FavoritesPurchase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;

/**
 * Defines the interface for favorites items.
 */
interface FavoritesItemInterface extends ContentEntityInterface, EntityChangedInterface {

  /**
   * Gets the parent favorites.
   *
   * @return \Drupal\commerce_favorites\Entity\FavoritesInterface|null
   *   The favorites, or NULL.
   */
  public function getFavorites();

  /**
   * Gets the parent favorites ID.
   *
   * @return int|null
   *   The favorites ID, or NULL.
   */
  public function getFavoritesId();

  /**
   * Gets the purchasable entity.
   *
   * @return \Drupal\commerce\PurchasableEntityInterface|null
   *   The purchasable entity, or NULL.
   */
  public function getPurchasableEntity();

  /**
   * Gets the purchasable entity ID.
   *
   * @return int
   *   The purchasable entity ID.
   */
  public function getPurchasableEntityId();

  /**
   * Gets the favorites item title.
   *
   * @return string
   *   The favorites item title
   */
  public function getTitle();

  /**
   * Gets the favorites item quantity.
   *
   * @return string
   *   The favorites item quantity
   */
  public function getQuantity();

  /**
   * Sets the favorites item quantity.
   *
   * @param string $quantity
   *   The favorites item quantity.
   *
   * @return $this
   */
  public function setQuantity($quantity);

  /**
   * Gets the favorites item comment.
   *
   * @return string
   *   The favorites item comment.
   */
  public function getComment();

  /**
   * Sets the favorites item comment.
   *
   * @param string $comment
   *   The favorites item comment.
   *
   * @return $this
   */
  public function setComment($comment);

  /**
   * Gets the favorites item priority.
   *
   * @return int
   *   The favorites item priority.
   */
  public function getPriority();

  /**
   * Sets the favorites item priority.
   *
   * @param int $priority
   *   The favorites item priority.
   *
   * @return $this
   */
  public function setPriority($priority);

  /**
   * Gets the purchases.
   *
   * Each object contains the order ID, quantity, and timestamp of a purchase.
   *
   * @return \Drupal\commerce_favorites\FavoritesPurchase[]
   *   The purchases.
   */
  public function getPurchases();

  /**
   * Sets the purchases.
   *
   * @param \Drupal\commerce_favorites\FavoritesPurchase[] $purchases
   *   The purchases.
   *
   * @return $this
   */
  public function setPurchases(array $purchases);

  /**
   * Adds a new purchase.
   *
   * @param \Drupal\commerce_favorites\FavoritesPurchase $purchase
   *   The purchase.
   */
  public function addPurchase(FavoritesPurchase $purchase);

  /**
   * Removes a purchase.
   *
   * @param \Drupal\commerce_favorites\FavoritesPurchase $purchase
   *   The purchase.
   *
   * @return $this
   */
  public function removePurchase(FavoritesPurchase $purchase);

  /**
   * Gets the purchased quantity.
   *
   * Represents the sum of all purchase quantities.
   *
   * @return string
   *   The purchased quantity.
   */
  public function getPurchasedQuantity();

  /**
   * Gets the timestamp of the last purchase.
   *
   * @return int|null
   *   The timestamp of the last purchase, or NULL if the favorites item
   *   hasn't been purchased yet.
   */
  public function getLastPurchasedTime();

  /**
   * Gets the favorites item creation timestamp.
   *
   * @return int
   *   The favorites item creation timestamp.
   */
  public function getCreatedTime();

  /**
   * Sets the favorites item creation timestamp.
   *
   * @param int $timestamp
   *   The favorites item creation timestamp.
   *
   * @return $this
   */
  public function setCreatedTime($timestamp);

}
