<?php

namespace Drupal\commerce_favorites;

use Drupal\commerce_favorites\Event\FavoritesAssignEvent;
use Drupal\commerce_favorites\Event\FavoritesEvents;
use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\UserInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Default favorites assignment implementation.
 */
class FavoritesAssignment implements FavoritesAssignmentInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The favorites manager.
   *
   * @var \Drupal\commerce_favorites\FavoritesManagerInterface
   */
  protected $favoritesManager;

  /**
   * Constructs a new FavoritesAssignment object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\commerce_favorites\FavoritesManagerInterface $favorites_manager
   *   The favorites manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EventDispatcherInterface $event_dispatcher, ConfigFactoryInterface $config_factory, FavoritesManagerInterface $favorites_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->eventDispatcher = $event_dispatcher;
    $this->configFactory = $config_factory;
    $this->favoritesManager = $favorites_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function assign(FavoritesInterface $favorites, UserInterface $account) {
    if (!empty($favorites->getOwnerId())) {
      // Skip favorites_lists which already have an owner.
      return;
    }

    $favorites->setOwner($account);
    // Update the referenced shipping profile.
    $shipping_profile = $favorites->getShippingProfile();
    if ($shipping_profile && empty($shipping_profile->getOwnerId())) {
      $shipping_profile->setOwner($account);
      $shipping_profile->save();
    }
    // Notify other modules.
    $event = new FavoritesAssignEvent($favorites, $account);
    $this->eventDispatcher->dispatch($event, FavoritesEvents::FAVORITES_ASSIGN);

    $favorites->save();
  }

  /**
   * {@inheritdoc}
   */
  public function assignMultiple(array $favorites_lists, UserInterface $account) {
    $allow_multiple = (bool) $this->configFactory->get('commerce_favorites.settings')->get('allow_multiple');
    /** @var \Drupal\commerce_favorites\FavoritesStorageInterface $favorites_storage */
    $favorites_storage = $this->entityTypeManager->getStorage('commerce_favorites');
    foreach ($favorites_lists as $favorites) {
      $default_favorites = $favorites_storage->loadDefaultByUser($account, $favorites->bundle());
      // Check if multiple favorites_lists are allowed, in which case we're assigning
      // the favorites to the given account.
      if ($allow_multiple || !$default_favorites) {
        $this->assign($favorites, $account);
        continue;
      }
      // In case a single favorites is allowed, we need to merge the favorites
      // items with the default favorites.
      $this->favoritesManager->merge($favorites, $default_favorites);
    }
  }

}
