<?php

namespace Drupal\commerce_favorites;

use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\commerce_favorites\Entity\FavoritesItemInterface;
use Drupal\commerce_favorites\Event\FavoritesEvents;
use Drupal\commerce_favorites\Event\FavoritesEmptyEvent;
use Drupal\commerce_favorites\Event\FavoritesEntityToggleEvent;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Default implementation of the favorites manager.
 *
 * Fires its own events, different from the favorites entity events by being a
 * result of user interaction (add to favorites form, favorites view, etc).
 */
class FavoritesManager implements FavoritesManagerInterface {

  /**
   * The favorites item storage.
   *
   * @var \Drupal\commerce_favorites\FavoritesItemStorageInterface
   */
  protected $favoritesItemStorage;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a new FavoritesManager object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EventDispatcherInterface $event_dispatcher) {
    $this->favoritesItemStorage = $entity_type_manager->getStorage('commerce_favorites_item');
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public function emptyFavorites(FavoritesInterface $favorites, $save_favorites = TRUE) {
    $favorites_items = $favorites->getItems();
    foreach ($favorites_items as $favorites_item) {
      $favorites_item->delete();
    }
    $favorites->setItems([]);

    $this->eventDispatcher->dispatch(new FavoritesEmptyEvent($favorites, $favorites_items), FavoritesEvents::FAVORITES_EMPTY);
    if ($save_favorites) {
      $favorites->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function addEntity(FavoritesInterface $favorites, PurchasableEntityInterface $entity, $quantity = 1, $combine = TRUE, $save_favorites = TRUE) {
    $favorites_item = $this->favoritesItemStorage->createFromPurchasableEntity($entity, [
      'quantity' => $quantity,
    ]);
    $purchasable_entity = $favorites_item->getPurchasableEntity();
    $matching_favorites_item = NULL;
    if ($combine) {
      $matching_favorites_item = $this->matchFavoritesItem($favorites_item, $favorites->getItems());
    }
    if ($matching_favorites_item) {
      $matching_favorites_item->setQuantity($quantity);
      $matching_favorites_item->save();
      $saved_favorites_item = $matching_favorites_item;
    }
    else {
      $favorites_item->save();
      $favorites->addItem($favorites_item);
      $saved_favorites_item = $favorites_item;
    }
    $event = new FavoritesEntityToggleEvent($favorites, $favorites_item);
    $this->eventDispatcher->dispatch($event, FavoritesEvents::FAVORITES_ENTITY_ADD);
    if ($save_favorites) {
      $favorites->save();
    }

    return $saved_favorites_item;
  }

  /**
   * {@inheritdoc}
   */
  public function merge(FavoritesInterface $source, FavoritesInterface $target, $save = TRUE) {
    foreach ($source->getItems() as $favorites_item) {
      $duplicate_favorites_item = $favorites_item->createDuplicate();
      $duplicate_favorites_item->save();
      $target->addItem($duplicate_favorites_item);
    }

    if ($save) {
      $target->save();
      $source->delete();
    }

    return $target;
  }

  /**
   * {@inheritdoc}
   */
  public function removeFavoritesItem(FavoritesInterface $favorites, FavoritesItemInterface $favorites_item, $save_favorites = TRUE) {
    $favorites->removeItem($favorites_item);
    if ($save_favorites) {
      $favorites->save();
    }
    $favorites_item->delete();

    $event = new FavoritesEntityToggleEvent($favorites, $favorites_item);
    $this->eventDispatcher->dispatch($event, FavoritesEvents::FAVORITES_ENTITY_REMOVE);
  }

  /**
   * Finds a matching favorites item for the given one.
   *
   * @param \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item
   *   The favorites item.
   * @param \Drupal\commerce_favorites\Entity\FavoritesItemInterface[] $favorites_items
   *   The favorites items to match against.
   *
   * @return \Drupal\commerce_favorites\Entity\FavoritesItemInterface|null
   *   A matching favorites item, or NULL if none was found.
   */
  protected function matchFavoritesItem(FavoritesItemInterface $favorites_item, array $favorites_items) {
    $matching_favorites_item = NULL;
    foreach ($favorites_items as $existing_favorites_item) {
      if ($existing_favorites_item->bundle() != $favorites_item->bundle()) {
        continue;
      }
      if ($existing_favorites_item->getPurchasableEntityId() != $favorites_item->getPurchasableEntityId()) {
        continue;
      }
      $matching_favorites_item = $existing_favorites_item;
      break;
    }

    return $matching_favorites_item;
  }

}
