<?php

namespace Drupal\commerce_favorites;

use Drupal\commerce_favorites\Exception\DuplicateFavoritesException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Default implementation of the favorites provider.
 */
class FavoritesProvider implements FavoritesProviderInterface {

  use StringTranslationTrait;

  /**
   * The favorites storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $favoritesStorage;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The session.
   *
   * @var \Drupal\commerce_favorites\FavoritesSessionInterface
   */
  protected $favoritesSession;

  /**
   * The loaded favorites data, keyed by favorites ID, then grouped by uid.
   *
   * @var array
   *
   * Each data item is an array with the following keys:
   * - type: The favorites type.
   *
   * Example:
   * @code
   * 1 => [
   *   10 => ['type' => 'default'],
   * ]
   * @endcode
   */
  protected $favoritesData = [];

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new FavoritesProvider object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\commerce_favorites\FavoritesSessionInterface $favorites_session
   *   The favorites session.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user, FavoritesSessionInterface $favorites_session, ConfigFactoryInterface $config_factory) {
    $this->favoritesStorage = $entity_type_manager->getStorage('commerce_favorites');
    $this->currentUser = $current_user;
    $this->favoritesSession = $favorites_session;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function createFavorites($favorites_type, AccountInterface $account = NULL, $name = NULL) {
    $account = $account ?: $this->currentUser;
    $uid = $account->id();
    $allow_multiple = $this->configFactory->get('commerce_favorites.settings')->get('allow_multiple');

    if (empty($allow_multiple) && $this->getFavoritesId($favorites_type, $account)) {
      // Don't allow multiple favorites entities matching the same criteria.
      throw new DuplicateFavoritesException("A favorites for type '$favorites_type' and account '$uid' already exists.");
    }

    // Create the new favorites entity.
    $favorites = $this->favoritesStorage->create([
      'type' => $favorites_type,
      'uid' => $uid,
      'name' => $name ?: $this->t('Favorites'),
      // @todo By now, we only support one favorites per user, so we automatically set it as the default one. In the long run we may need to distinct here.
      'is_default' => TRUE,
    ]);
    $favorites->save();
    // Store the new favorites id in the anonymous user's session so that it can
    // be retrieved on the next page load.
    if ($account->isAnonymous()) {
      $this->favoritesSession->addFavoritesId($favorites->id());
    }
    // Favorites data has already been loaded, add the new favorites to the list.
    if (isset($this->favoritesData[$uid])) {
      $this->favoritesData[$uid][$favorites->id()] = [
        'type' => $favorites_type,
      ];
    }

    return $favorites;
  }

  /**
   * {@inheritdoc}
   */
  public function getFavorites($favorites_type, AccountInterface $account = NULL) {
    $favorites = NULL;
    $favorites_id = $this->getFavoritesId($favorites_type, $account);
    if ($favorites_id) {
      $favorites = $this->favoritesStorage->load($favorites_id);
    }

    return $favorites;
  }

  /**
   * {@inheritdoc}
   */
  public function getFavoritesId($favorites_type, AccountInterface $account = NULL) {
    $favorites_id = NULL;
    $favorites_data = $this->loadFavoritesData($account);
    if ($favorites_data) {
      $search = [
        'type' => $favorites_type,
      ];
      $favorites_id = array_search($search, $favorites_data);
    }

    return $favorites_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getFavoritesLists(AccountInterface $account = NULL) {
    $favorites_lists = [];
    $favorites_ids = $this->getFavoritesIds($account);
    if ($favorites_ids) {
      $favorites_lists = $this->favoritesStorage->loadMultiple($favorites_ids);
    }

    return $favorites_lists;
  }

  /**
   * {@inheritdoc}
   */
  public function getFavoritesIds(AccountInterface $account = NULL) {
    $favorites_data = $this->loadFavoritesData($account);
    return array_keys($favorites_data);
  }

  /**
   * Loads the favorites data for the given user.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user. If empty, the current user is assumed.
   *
   * @return array
   *   The favorites data.
   */
  protected function loadFavoritesData(AccountInterface $account = NULL) {
    $account = $account ?: $this->currentUser;
    $uid = $account->id();
    if (isset($this->favoritesData[$uid])) {
      return $this->favoritesData[$uid];
    }

    if ($account->isAuthenticated()) {
      $query = $this->favoritesStorage->getQuery()
        ->condition('uid', $account->id())
        ->sort('is_default', 'DESC')
        ->sort('favorites_id', 'DESC')
        ->accessCheck(FALSE);
      $favorites_ids = $query->execute();
    }
    else {
      $favorites_ids = $this->favoritesSession->getFavoritesIds();
    }

    $this->favoritesData[$uid] = [];
    if (!$favorites_ids) {
      return [];
    }
    // Getting the favorites data and validating the favorites ids received from
    // the session requires loading the entities. This is a performance hit, but
    // it's assumed that these entities would be loaded at one point anyway.
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface[] $favorites_lists */
    $favorites_lists = $this->favoritesStorage->loadMultiple($favorites_ids);
    foreach ($favorites_lists as $favorites) {
      if ($favorites->getOwnerId() != $uid) {
        // Skip favorites_lists that are no longer eligible.
        continue;
      }

      $this->favoritesData[$uid][$favorites->id()] = [
        'type' => $favorites->bundle(),
      ];
    }

    return $this->favoritesData[$uid];
  }

}
