<?php

namespace Drupal\commerce_favorites\Form;

use Drupal\commerce\AjaxFormTrait;
use Drupal\commerce\Context;
use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Resolver\OrderTypeResolverInterface;
use Drupal\commerce_price\Resolver\ChainPriceResolverInterface;
use Drupal\commerce_store\CurrentStoreInterface;
use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\commerce_favorites\Entity\FavoritesItemInterface;
use Drupal\commerce_favorites\FavoritesManagerInterface;
use Drupal\commerce_favorites\FavoritesSessionInterface;

use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Block\BlockManagerInterface;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the favorites user form.
 *
 * Used for both the canonical ("/favorites/{code}") and user-form
 * ("/user/{user}/favorites/{commerce_favorites}") pages.
 */
class FavoritesUserForm extends EntityForm {

  use AjaxFormTrait;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  /**
   * The cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The current store.
   *
   * @var \Drupal\commerce_store\CurrentStoreInterface
   */
  protected $currentStore;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The order type resolver.
   *
   * @var \Drupal\commerce_order\Resolver\OrderTypeResolverInterface
   */
  protected $orderTypeResolver;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The favorites settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * The chain base price resolver.
   *
   * @var \Drupal\commerce_price\Resolver\ChainPriceResolverInterface
   */
  protected $chainPriceResolver;

  /**
   * The favorites manager.
   *
   * @var \Drupal\commerce_favorites\FavoritesManagerInterface
   */
  protected $favoritesManager;

  /**
   * The favorites session.
   *
   * @var \Drupal\commerce_favorites\FavoritesSessionInterface
   */
  protected $favoritesSession;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The block manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * Constructs a new FavoritesUserForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\commerce_cart\CartManagerInterface $cart_manager
   *   The cart manager.
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   The cart provider.
   * @param \Drupal\commerce_store\CurrentStoreInterface $current_store
   *   The current store.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\commerce_order\Resolver\OrderTypeResolverInterface $order_type_resolver
   *   The order type resolver.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\commerce_price\Resolver\ChainPriceResolverInterface $chain_price_resolver
   *   The price resolver.
   * @param \Drupal\commerce_favorites\FavoritesManagerInterface $favorites_manager
   *   The favorites manager.
   * @param \Drupal\commerce_favorites\FavoritesSessionInterface $favorites_session
   *   The favorites session.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory, CartManagerInterface $cart_manager, CartProviderInterface $cart_provider, CurrentStoreInterface $current_store, AccountInterface $current_user, OrderTypeResolverInterface $order_type_resolver, RouteMatchInterface $route_match, ChainPriceResolverInterface $chain_price_resolver, FavoritesManagerInterface $favorites_manager, FavoritesSessionInterface $favorites_session, LanguageManagerInterface $language_manager, BlockManagerInterface $block_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
    $this->currentStore = $current_store;
    $this->currentUser = $current_user;
    $this->orderTypeResolver = $order_type_resolver;
    $this->routeMatch = $route_match;
    $this->settings = $config_factory->get('commerce_favorites.settings');
    $this->chainPriceResolver = $chain_price_resolver;
    $this->favoritesManager = $favorites_manager;
    $this->favoritesSession = $favorites_session;
    $this->languageManager = $language_manager;
    $this->blockManager = $block_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_store.current_store'),
      $container->get('current_user'),
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('current_route_match'),
      $container->get('commerce_price.chain_price_resolver'),
      $container->get('commerce_favorites.favorites_manager'),
      $container->get('commerce_favorites.favorites_session'),
      $container->get('language_manager'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites */
    $favorites = $this->entity;
    $owner_access = $this->ownerAccess($favorites);
    $favorites_has_items = $favorites->hasItems();

    $form['#tree'] = TRUE;
    $form['#process'][] = '::processForm';
    $form['#theme'] = 'commerce_favorites_user_form';
    $form['#attached']['library'][] = 'commerce_favorites/user';
    // Workaround for core bug #2897377.
    $form['#id'] = Html::getId($form_state->getBuildInfo()['form_id']);

    $form['header'] = [
      '#type' => 'container',
    ];
    $form['header']['empty_text'] = [
      '#markup' => $this->t('Your favorites list is empty.'),
      '#access' => !$favorites_has_items,
    ];
    $form['header']['add_all_to_cart'] = [
      '#type' => 'submit',
      '#value' => t('Add the entire list to cart'),
      '#ajax' => [
        'callback' => [get_called_class(), 'ajaxRefreshForm'],
      ],
      '#access' => $favorites_has_items,
    ];
    $form['header']['share'] = [
      '#type' => 'link',
      '#title' => $this->t('Share the list by email'),
      '#url' => $favorites->toUrl('share-form', [
        'language' => $this->languageManager->getCurrentLanguage(),
      ]),
      '#attributes' => [
        'class' => [
          'use-ajax',
          'button',
          'btn',
          'btn-default',
          'favorites-button',
        ],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => Json::encode([
          'width' => 700,
          'title' => $this->t('Share the list by email'),
        ]),
        'role' => 'button',
      ],
      '#access' => $owner_access && $favorites_has_items,
    ];

    $form['items'] = [];
    foreach ($favorites->getItems() as $item) {
      $notes_action_text = 'Add a Note';
      if ($item->getComment()) {
        $notes_action_text = 'Edit Note';
      }

      $purchasable_entity = $item->getPurchasableEntity();
      if (!$purchasable_entity || !$purchasable_entity->access('view')) {
        continue;
      }
      $item_form = &$form['items'][$item->id()];

      $item_form = [
        '#type' => 'container',
      ];
      $item_form['entity'] = $this->renderPurchasableEntity($purchasable_entity);
      $item_form['details'] = [
        '#theme' => 'commerce_favorites_item_details',
        '#favorites_item_entity' => $item,
      ];
      $item_form['details_edit'] = [
        '#type' => 'link',
        '#title' => $this->t($notes_action_text),
        '#url' => $item->toUrl('details-form'),
        '#attributes' => [
          'class' => [
            'use-ajax',
            'favorites-item__details-edit-link',
          ],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => Json::encode([
            'width' => 700,
            'title' => $this->t($notes_action_text),
          ]),
        ],
        '#access' => $owner_access,
      ];
      $item_form['actions'] = [
        '#type' => 'container',
      ];
      $item_form['actions']['add_to_cart'] = [
        '#type' => 'submit',
        '#value' => t('Add to Cart'),
        '#attributes' => Array(
          'class' => Array('favorites-add-to-cart-button','button--add-to-cart','button','button--primary'),
        ),
        '#ajax' => [
          'event' => 'click',
          // 'callback' => [get_called_class(), 'ajaxRefreshForm'],
          'callback' => '::ajaxRefreshFormAndCartBlock',
        ],
        '#submit' => [
          '::addToCartSubmit',
        ],
        '#name' => 'add-to-cart-' . $item->id(),
        '#item_id' => $item->id(),
      ];
      $item_form['actions']['remove'] = [
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#attributes' => Array(
          'class' => Array('favorites-remove-from-favorites-button')
        ),
        '#ajax' => [
          'event' => 'click',
          // 'callback' => [get_called_class(), 'ajaxRefreshForm'],
          'callback' => '::ajaxRefreshFormAndFavoritesBlock',
        ],
        '#submit' => [
          '::removeItem',
        ],
        '#name' => 'remove-' . $item->id(),
        '#access' => $owner_access,
        '#item_id' => $item->id(),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actionsElement(array $form, FormStateInterface $form_state) {
    return [];
  }

  /**
   * Submit callback for the "Add to cart" button.
   */
  public function addToCartSubmit(array $form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $favorites_item_storage = $this->entityTypeManager->getStorage('commerce_favorites_item');
    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
    $favorites_item = $favorites_item_storage->load($triggering_element['#item_id']);
    $this->addItemToCart($favorites_item);
  }

  /**
   * Submit callback for the "Remove" button.
   */
  public function removeItem(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites */
    $favorites = $this->entity;
    $triggering_element = $form_state->getTriggeringElement();
    $favorites_item_storage = $this->entityTypeManager->getStorage('commerce_favorites_item');
    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
    $favorites_item = $favorites_item_storage->load($triggering_element['#item_id']);
    $this->favoritesManager->removeFavoritesItem($favorites, $favorites_item);

    $this->messenger()->addStatus($this->t('@entity has been removed from your Favorites.', [
      '@entity' => $favorites_item->label(),
    ]));
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites */
    $favorites = $this->entity;
    foreach ($favorites->getItems() as $favorites_item) {
      $this->addItemToCart($favorites_item);
    }
  }

  /**
   * Renders the given purchasable entity.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $purchasable_entity
   *   The purchasable entity.
   *
   * @return array
   *   The render array.
   */
  protected function renderPurchasableEntity(PurchasableEntityInterface $purchasable_entity) {
    $entity_type_id = $purchasable_entity->getEntityTypeId();
    $view_builder = $this->entityTypeManager->getViewBuilder($entity_type_id);
    $view_mode = $this->settings->get('view_modes.' . $entity_type_id);
    $view_mode = $view_mode ?: 'cart';
    $build = $view_builder->view($purchasable_entity, $view_mode);

    return $build;
  }

  /**
   * Checks whether the current user owns the given favorites.
   *
   * Used to determine whether the user is allowed to modify and share
   * the favorites.
   *
   * @param \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites
   *   The favorites.
   *
   * @return bool
   *   TRUE if the current user owns the given favorites, FALSE otherwise.
   */
  protected function ownerAccess(FavoritesInterface $favorites) {
    if ($this->currentUser->isAnonymous()) {
      // Anonymous favorites_lists aren't fully implemented yet.
      return $this->favoritesSession->hasFavoritesId($favorites->id());
    }
    if ($favorites->getOwnerId() != $this->currentUser->id()) {
      return FALSE;
    }
    if ($this->routeMatch->getRouteName() != 'entity.commerce_favorites.user_form') {
      // Users should only modify their favorites_lists via the user form.
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Adds a favorites item to the cart.
   *
   * @param \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item
   *   The favorites item to move to the cart.
   */
  protected function addItemToCart(FavoritesItemInterface $favorites_item) {
    $purchasable_entity = $favorites_item->getPurchasableEntity();
    /** @var \Drupal\commerce_order\OrderItemStorageInterface $order_item_storage */
    $order_item_storage = $this->entityTypeManager->getStorage('commerce_order_item');
    $values = [
      'quantity' => $favorites_item->getQuantity(),
    ];
    $order_item = $order_item_storage->createFromPurchasableEntity($purchasable_entity, $values);
    $order_type_id = $this->orderTypeResolver->resolve($order_item);
    $store = $this->selectStore($purchasable_entity);
    $cart = $this->cartProvider->getCart($order_type_id, $store);
    if (!$order_item->isUnitPriceOverridden()) {
      $context = new Context($this->currentUser, $store);
      $resolved_price = $this->chainPriceResolver->resolve($purchasable_entity, $order_item->getQuantity(), $context);
      $order_item->setUnitPrice($resolved_price);
    }
    if (!$cart) {
      $cart = $this->cartProvider->createCart($order_type_id, $store);
    }
    $this->cartManager->addOrderItem($cart, $order_item, TRUE);

    // JBB - 2023-12-12 - add this status update because we won't show the "Add to cart" modal
    $this->messenger()->addStatus($this->t('@entity has been added to <a href="/cart">your Cart</a>.', [
      '@entity' => $favorites_item->label(),
    ]));
  }

  /**
   * Selects the store for the given purchasable entity.
   *
   * Copied over from AddToCartForm.
   *
   * If the entity is sold from one store, then that store is selected.
   * If the entity is sold from multiple stores, and the current store is
   * one of them, then that store is selected.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $entity
   *   The entity being added to cart.
   *
   * @throws \Exception
   *   When the entity can't be purchased from the current store.
   *
   * @return \Drupal\commerce_store\Entity\StoreInterface
   *   The selected store.
   */
  protected function selectStore(PurchasableEntityInterface $entity) {
    $stores = $entity->getStores();
    if (count($stores) === 1) {
      $store = reset($stores);
    }
    elseif (count($stores) === 0) {
      // Malformed entity.
      throw new \Exception('The given entity is not assigned to any store.');
    }
    else {
      $store = $this->currentStore->getStore();
      if (!in_array($store, $stores)) {
        // Indicates that the site listings are not filtered properly.
        throw new \Exception("The given entity can't be purchased from the current store.");
      }
    }

    return $store;
  }

  /**
   * Ajax handler for refreshing the entire form as well as the cart block.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   */
  public function ajaxRefreshFormAndCartBlock(array $form, FormStateInterface $form_state) {
    $response = AjaxFormTrait::ajaxRefreshForm($form, $form_state);
    $cart_block = $this->blockManager->createInstance('commerce_cart', []);
    $response->addCommand(new ReplaceCommand('.cart--cart-block', $cart_block->build()));
    return $response;
  }

  /**
   * Ajax handler for refreshing the entire form as well as the favorites block.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   */
  public function ajaxRefreshFormAndFavoritesBlock(array $form, FormStateInterface $form_state) {
    $response = AjaxFormTrait::ajaxRefreshForm($form, $form_state);
    $favorites_block = $this->blockManager->createInstance('commerce_favorites', []);
    $response->addCommand(new ReplaceCommand('.favorites-block', $favorites_block->build()));
    return $response;
  }

}
