<?php

namespace Drupal\commerce_favorites\Plugin\Block;

use Drupal\commerce_favorites\FavoritesProviderInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a favorites block.
 *
 * @Block(
 *   id = "commerce_favorites",
 *   admin_label = @Translation("Favorites"),
 *   category = @Translation("Commerce")
 * )
 */
class FavoritesBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The favorites provider.
   *
   * @var \Drupal\commerce_favorites\FavoritesProviderInterface
   */
  protected $favoritesProvider;

  /**
   * Constructs a new FavoritesBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_favorites\FavoritesProviderInterface $favorites_provider
   *   The favorites provider.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FavoritesProviderInterface $favorites_provider) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->favoritesProvider = $favorites_provider;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('commerce_favorites.favorites_provider')
    );
  }

  /**
   * Builds the favorites block.
   *
   * @return array
   *   A render array.
   */
  public function build() {
    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface[] $favorites_lists */
    $favorites = $this->favoritesProvider->getFavorites('default');
    $count = $favorites ? count($favorites->getItems()) : 0;

    return [
      '#theme' => 'commerce_favorites_block',
      '#count' => $count,
      '#count_text' => $this->formatPlural($count, '@count item', '@count items', [], ['context' => 'favorites block']),
      '#favorites_entity' => $favorites,
      '#url' => Url::fromRoute('commerce_favorites.page'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return Cache::mergeContexts(parent::getCacheContexts(), ['favorites']);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $cache_tags = parent::getCacheTags();
    $favorites_cache_tags = [];

    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface[] $favorites_lists */
    $favorites_lists = $this->favoritesProvider->getFavoritesLists();
    foreach ($favorites_lists as $favorites) {
      // Add tags for all favorites_lists regardless items.
      $favorites_cache_tags = Cache::mergeTags($favorites_cache_tags, $favorites->getCacheTags());
    }
    return Cache::mergeTags($cache_tags, $favorites_cache_tags);
  }

}
