<?php

namespace Drupal\commerce_favorites\Plugin\views\field;

use Drupal\commerce_favorites\FavoritesManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Plugin\views\field\UncacheableFieldHandlerTrait;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form element for removing the favorites item.
 *
 * @ViewsField("commerce_favorites_item_remove_button")
 */
class RemoveButton extends FieldPluginBase {

  use UncacheableFieldHandlerTrait;

  /**
   * The favorites manager.
   *
   * @var \Drupal\commerce_favorites\FavoritesManagerInterface
   */
  protected $favoritesManager;

  /**
   * Constructs a new EditRemove object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_favorites\FavoritesManagerInterface $favorites_manager
   *   The favorites manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FavoritesManagerInterface $favorites_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->favoritesManager = $favorites_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('commerce_favorites.favorites_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function clickSortable() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getValue(ResultRow $row, $field = NULL) {
    return '<!--form-item-' . $this->options['id'] . '--' . $row->index . '-->';
  }

  /**
   * Form constructor for the views form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function viewsForm(array &$form, FormStateInterface $form_state) {
    // Make sure we do not accidentally cache this form.
    $form['#cache']['max-age'] = 0;
    // The view is empty, abort.
    if (empty($this->view->result)) {
      unset($form['actions']);
      return;
    }

    $form[$this->options['id']]['#tree'] = TRUE;
    foreach ($this->view->result as $row_index => $row) {
      $form[$this->options['id']][$row_index] = [
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#name' => 'delete-favorites-item-' . $row_index,
        '#remove_favorites_item' => TRUE,
        '#row_index' => $row_index,
        '#attributes' => ['class' => ['delete-favorites-item']],
      ];
    }
  }

  /**
   * Submit handler for the views form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function viewsFormSubmit(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    if (!empty($triggering_element['#remove_favorites_item'])) {
      $row_index = $triggering_element['#row_index'];
      /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
      $favorites_item = $this->getEntity($this->view->result[$row_index]);
      $this->favoritesManager->removeFavoritesItem($favorites_item->getFavorites(), $favorites_item);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing.
  }

}
