<?php

namespace Drupal\Tests\commerce_favorites\Functional;

use Drupal\commerce_price\Price;
use Drupal\commerce_product\Entity\ProductVariationType;
use Drupal\commerce_favorites\Entity\FavoritesItem;
use Drupal\Core\Url;

/**
 * Tests the favorites item UI.
 *
 * @group commerce_favorites
 */
class FavoritesItemTest extends FavoritesBrowserTestBase {

  /**
   * A test favorites.
   *
   * @var \Drupal\commerce_favorites\Entity\FavoritesInterface
   */
  protected $favorites;

  /**
   * A test variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $firstVariation;

  /**
   * A test variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $secondVariation;

  /**
   * The favorites item collection uri.
   *
   * @var string
   */
  protected $favoritesItemCollectionUri;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Turn off title generation to allow explicit values to be used.
    $variation_type = ProductVariationType::load('default');
    $variation_type->setGenerateTitle(FALSE);
    $variation_type->save();

    $this->favorites = $this->createEntity('commerce_favorites', [
      'type' => 'default',
      'name' => 'Secret gifts',
      'public' => TRUE,
      'keep_purchased_items' => TRUE,
    ]);

    $this->firstVariation = $this->createEntity('commerce_product_variation', [
      'type' => 'default',
      'sku' => 'S-DESK',
      'title' => 'Standing desk',
      'price' => new Price('120.00', 'USD'),
    ]);
    $this->secondVariation = $this->createEntity('commerce_product_variation', [
      'type' => 'default',
      'sku' => 'R-DESK',
      'title' => 'Regular desk',
      'price' => new Price('70.00', 'USD'),
    ]);

    $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => $this->randomMachineName(),
      'stores' => [$this->store],
      'variations' => [$this->firstVariation, $this->secondVariation],
    ]);
    $this->favoritesItemCollectionUri = Url::fromRoute('entity.commerce_favorites_item.collection', [
      'commerce_favorites' => $this->favorites->id(),
    ])->toString();
  }

  /**
   * Tests adding a favorites item.
   */
  public function testAdd() {
    $this->drupalGet($this->favoritesItemCollectionUri);
    $this->clickLink('Add item');

    $this->submitForm([
      'purchasable_entity[0][target_id]' => 'Standing desk (1)',
      'quantity[0][value]' => '1',
      'comment[0][value]' => 'Love this desk',
    ], 'Save');
    $this->assertSession()->pageTextContains('The item Standing desk has been successfully saved.');

    $favorites_item = FavoritesItem::load(1);
    $this->assertEquals($this->favorites->id(), $favorites_item->getfavoritesId());
    $this->assertEquals($this->firstVariation->id(), $favorites_item->getPurchasableEntityId());
    $this->assertEquals('1', $favorites_item->getQuantity());
    $this->assertNotEmpty($favorites_item->getComment());
  }

  /**
   * Tests editing a favorites item.
   */
  public function testEdit() {
    $favorites_item = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'favorites_id' => $this->favorites->id(),
      'purchasable_entity' => $this->firstVariation->id(),
      'quantity' => '1',
    ]);
    $this->drupalGet($favorites_item->toUrl('edit-form'));
    $this->submitForm([
      'purchasable_entity[0][target_id]' => 'Regular desk (2)',
      'quantity[0][value]' => '5',
      'comment[0][value]' => 'My updated comment',
    ], 'Save');
    $this->assertSession()->pageTextNotContains('The item Regular desk has been successfully saved.');

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites_item')->resetCache([$favorites_item->id()]);
    $favorites_item = FavoritesItem::load(1);

    // It is not possible to change product variation on edit.
    $this->assertNotEquals($this->secondVariation->id(), $favorites_item->getPurchasableEntityId());
    $this->assertEqual($favorites_item->getComment(), 'My updated comment');
    $this->assertEquals('5', $favorites_item->getQuantity());
  }

  /**
   * Tests duplicating a favorites item.
   */
  public function testDuplicate() {
    $favorites_item = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'favorites_id' => $this->favorites->id(),
      'purchasable_entity' => $this->firstVariation->id(),
      'quantity' => '2',
    ]);
    $this->drupalGet($favorites_item->toUrl('duplicate-form'));
    $this->assertSession()->pageTextContains('Duplicate Standing desk');
    $this->submitForm([
      'purchasable_entity[0][target_id]' => 'Regular desk (2)',
      'quantity[0][value]' => '5',
      'comment[0][value]' => 'Still regular is cheaper',
    ], 'Save');
    $this->assertSession()->pageTextContains('The item Regular desk has been successfully saved.');

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites_item')->resetCache([$favorites_item->id()]);
    // Confirm that the original favorites item is unchanged.
    $favorites_item_1 = FavoritesItem::load(1);
    $this->assertEquals($this->favorites->id(), $favorites_item_1->getfavoritesId());
    $this->assertEquals($this->firstVariation->id(), $favorites_item_1->getPurchasableEntityId());
    $this->assertEquals('2', $favorites_item_1->getQuantity());
    $this->assertEmpty($favorites_item_1->getComment());

    // Confirm that the new favorites item has the expected data.
    $favorites_item_2 = FavoritesItem::load(2);
    $this->assertEquals($this->favorites->id(), $favorites_item_2->getfavoritesId());
    $this->assertEquals($this->secondVariation->id(), $favorites_item_2->getPurchasableEntityId());
    $this->assertEquals('5', $favorites_item_2->getQuantity());
    $this->assertNotEmpty($favorites_item_2->getComment());
    $this->assertEqual($favorites_item_2->getComment(), 'Still regular is cheaper');
  }

  /**
   * Tests deleting a favorites item.
   */
  public function testDelete() {
    $favorites_item = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'favorites_id' => $this->favorites->id(),
      'purchasable_entity' => $this->firstVariation->id(),
      'quantity' => '1',
      'comment' => 'You are gonna delete me',
    ]);
    $this->drupalGet($favorites_item->toUrl('delete-form'));
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('This action cannot be undone.');
    $this->submitForm([], t('Delete'));

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites_item')->resetCache([$favorites_item->id()]);
    $favorites_item_exists = (bool) FavoritesItem::load($favorites_item->id());
    $this->assertFalse($favorites_item_exists);
  }

}
