<?php

namespace Drupal\Tests\commerce_favorites\Functional;

use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\Core\Url;

/**
 * Tests the favorites UI.
 *
 * @group commerce_favorites
 */
class FavoritesTest extends FavoritesBrowserTestBase {

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->config('commerce_favorites.settings')->set('allow_multiple', 1)->save();
  }

  /**
   * Tests adding a favorites.
   */
  public function testAdd() {
    $this->drupalGet(Url::fromRoute('entity.commerce_favorites.collection')->toString());
    $this->clickLink('Add favorites');

    $this->submitForm([
      'name[0][value]' => 'Special desire',
      'is_public[value]' => '1',
      'keep_purchased_items[value]' => '1',
    ], 'Save');
    $this->assertSession()->pageTextContains('The favorites Special desire has been successfully saved.');

    $favorites = Favorites::load(1);
    $this->assertEquals('Special desire', $favorites->getName());
    $this->assertTrue($favorites->isPublic());
    $this->assertTrue($favorites->getKeepPurchasedItems());
    $this->assertEmpty($favorites->getItems());
    $this->assertNotEmpty($favorites->getCode());
  }

  /**
   * Tests adding multiple favorites.
   */
  public function testAddMultiple() {
    $this->drupalGet(Url::fromRoute('entity.commerce_favorites.collection')->toString());
    $this->clickLink('Add favorites');

    $this->submitForm([
      'name[0][value]' => 'Special desire',
      'is_public[value]' => '1',
      'keep_purchased_items[value]' => '1',
    ], 'Save');
    $this->assertSession()->pageTextContains('The favorites Special desire has been successfully saved.');

    $favorites = Favorites::load(1);
    $this->assertEquals('Special desire', $favorites->getName());
    $this->assertTrue($favorites->isPublic());
    $this->assertTrue($favorites->getKeepPurchasedItems());
    $this->assertEmpty($favorites->getItems());
    $this->assertNotEmpty($favorites->getCode());

    // Add second favorites.
    $this->drupalGet(Url::fromRoute('entity.commerce_favorites.collection')->toString());
    $this->clickLink('Add favorites');
    $this->submitForm([
      'name[0][value]' => 'Special desire 2',
      'is_public[value]' => '1',
      'keep_purchased_items[value]' => '1',
    ], 'Save');

    $this->assertSession()->pageTextContains('The favorites Special desire 2 has been successfully saved.');

    $favorites = Favorites::load(2);
    $this->assertEquals('Special desire 2', $favorites->getName());
    $this->assertTrue($favorites->isPublic());
    $this->assertTrue($favorites->getKeepPurchasedItems());
    $this->assertEmpty($favorites->getItems());
    $this->assertNotEmpty($favorites->getCode());

    // Change settings and don't allow multiple favoritess.
    $this->config('commerce_favorites.settings')->set('allow_multiple', 0)->save();
    $this->drupalGet(Url::fromRoute('entity.commerce_favorites.collection')->toString());
    $this->clickLink('Add favorites');
    $this->submitForm([
      'name[0][value]' => 'Special desire 3',
      'is_public[value]' => '1',
      'keep_purchased_items[value]' => '1',
    ], 'Save');

    $this->assertSession()->pageTextContains('Cannot create a new favorites (Only a single favorites per customer is allowed).');

  }

  /**
   * Tests editing a favorites.
   */
  public function testEdit() {
    $favorites = $this->createEntity('commerce_favorites', [
      'type' => 'default',
      'name' => $this->randomMachineName(8),
      'is_public' => TRUE,
      'keep_purchased_items' => FALSE,
    ]);
    $code = $favorites->getCode();
    $this->drupalGet($favorites->toUrl('edit-form'));
    $favorites_tab_uri = Url::fromRoute('entity.commerce_favorites_item.collection', [
      'commerce_favorites' => $favorites->id(),
    ])->toString();
    $this->assertSession()->linkByHrefExists($favorites->toUrl('edit-form')->toString());
    $this->assertSession()->linkByHrefExists($favorites->toUrl('duplicate-form')->toString());
    $this->assertSession()->linkByHrefExists($favorites_tab_uri);

    $this->submitForm([
      'name[0][value]' => 'Top secret gifts',
      'is_public[value]' => '0',
      'keep_purchased_items[value]' => '1',
    ], 'Save');
    $this->assertSession()->pageTextContains('The favorites Top secret gifts has been successfully saved');

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites')->resetCache([$favorites->id()]);
    $favorites = Favorites::load(1);
    $this->assertEquals('Top secret gifts', $favorites->getName());
    $this->assertFalse($favorites->isPublic());
    $this->assertTrue($favorites->getKeepPurchasedItems());
    $this->assertEmpty($favorites->hasItems());
    $this->assertEquals($code, $favorites->getCode());
  }

  /**
   * Tests duplicating a favorites.
   */
  public function testDuplicate() {
    $favorites = $this->createEntity('commerce_favorites', [
      'type' => 'default',
      'name' => 'Secret gifts',
      'is_public' => TRUE,
      'keep_purchased_items' => TRUE,
    ]);
    $this->drupalGet($favorites->toUrl('duplicate-form'));
    $this->assertSession()->pageTextContains('Duplicate Secret gifts');
    $favorites_tab_uri = Url::fromRoute('entity.commerce_favorites_item.collection', [
      'commerce_favorites' => $favorites->id(),
    ])->toString();
    $this->assertSession()->linkByHrefExists($favorites->toUrl('edit-form')->toString());
    $this->assertSession()->linkByHrefExists($favorites->toUrl('duplicate-form')->toString());
    $this->assertSession()->linkByHrefExists($favorites_tab_uri);
    $this->submitForm([
      'name[0][value]' => 'Secret gits duplicated',
      'is_public[value]' => '0',
    ], 'Save');
    $this->assertSession()->pageTextContains('The favorites Secret gits duplicated has been successfully saved.');

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites')->resetCache([$favorites->id()]);
    // Confirm that the original favorites is unchanged.
    $favorites_1 = Favorites::load(1);
    $this->assertEquals('Secret gifts', $favorites_1->getName());
    $this->assertTrue($favorites_1->isPublic());

    // Confirm that the new favorites has the expected data.
    $favorites_2 = Favorites::load(2);
    $this->assertEquals('Secret gits duplicated', $favorites_2->getName());
    $this->assertNotTrue($favorites_2->isPublic());
    // Check that there is no attached items.
    $this->assertEmpty($favorites_2->hasItems());

    // Check that codes are different.
    $this->assertNotSame($favorites_1->getCode(), $favorites_2->getCode());
  }

  /**
   * Tests deleting a favorites.
   */
  public function testDelete() {
    $favorites = $this->createEntity('commerce_favorites', [
      'type' => 'default',
      'name' => $this->randomMachineName(8),
    ]);
    $this->drupalGet($favorites->toUrl('delete-form'));
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('This action cannot be undone.');
    $this->submitForm([], t('Delete'));

    \Drupal::service('entity_type.manager')->getStorage('commerce_favorites')->resetCache([$favorites->id()]);
    $favorites_exists = (bool) Favorites::load($favorites->id());
    $this->assertFalse($favorites_exists);
  }

}
