<?php

namespace Drupal\Tests\commerce_favorites\FunctionalJavascript;

use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\Entity\FavoritesType;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Tests\commerce\FunctionalJavascript\CommerceWebDriverTestBase;

/**
 * Tests the favorites anonymous access.
 *
 * @group commerce_favorites
 */
class FavoritesAnonymousTest extends CommerceWebDriverTestBase {

  /**
   * The favorites.
   *
   * @var \Drupal\commerce_favorites\Entity\FavoritesInterface
   */
  protected $favorites;

  /**
   * A product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $variation1;

  /**
   * A product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $variation2;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'commerce_product',
    'commerce_cart',
    'commerce_favorites',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $form_display = EntityViewDisplay::load('commerce_product.default.default');
    $form_display->setComponent('variations', [
      'type' => 'commerce_add_to_cart',
      'weight' => 1,
      'label' => 'hidden',
      'settings' => [
        'combine' => TRUE,
      ],
      'third_party_settings' => [
        'commerce_favorites' => [
          'show_favorites' => TRUE,
          'weight_favorites' => 99,
          'label_favorites' => 'Add to favorites',
          'region' => 'content',
        ],
      ],
    ]);
    $form_display->save();

    $this->createEntity('commerce_product_variation_type', [
      'id' => 'test',
      'label' => 'Test',
      'orderItemType' => 'default',
      'generateTitle' => FALSE,
    ]);
    $entity_display = commerce_get_entity_display('commerce_product_variation', 'test', 'view');
    $entity_display->setComponent('title', [
      'label' => 'above',
      'type' => 'string',
    ]);
    $entity_display->save();

    $this->variation1 = $this->createEntity('commerce_product_variation', [
      'type' => 'test',
      'title' => 'First variation',
      'sku' => strtolower($this->randomMachineName()),
      'price' => [
        'number' => 9.99,
        'currency_code' => 'USD',
      ],
    ]);
    $this->variation2 = $this->createEntity('commerce_product_variation', [
      'type' => 'test',
      'title' => 'Second variation',
      'sku' => strtolower($this->randomMachineName()),
      'price' => [
        'number' => 20.99,
        'currency_code' => 'USD',
      ],
    ]);
    /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
    $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'My product',
      'variations' => [$this->variation1, $this->variation2],
      'stores' => [$this->store],
    ]);
  }

  /**
   * Test add to favorites access.
   */
  public function testNotAllowed() {
    $this->drupalGet('product/1');
    $this->assertSession()->buttonExists('Add to favorites');

    $this->drupalLogout();
    drupal_flush_all_caches();

    $this->drupalGet('product/1');
    $this->assertSession()->buttonNotExists('Add to favorites');
  }

  /**
   * Tests adding favorites.
   */
  public function testAddFavorites() {
    // Allow adding favorites for anonymous user.
    $favorites_type = FavoritesType::load('default');
    $favorites_type->setAllowAnonymous(TRUE);
    $favorites_type->save();
    $this->reloadEntity($favorites_type);

    $this->drupalLogout();
    $this->drupalGet('product/1');
    $this->getSession()->getPage()->hasButton('edit-favorites');
    $this->getSession()->getPage()->findButton('Add to favorites')->click();
    $this->assertSession()->assertWaitOnAjaxRequest();

    $this->drupalGet('favorites');
    $this->assertSession()->elementExists('css', 'input[data-drupal-selector="edit-header-add-all-to-cart"]');
    $this->assertSession()->elementExists('css', 'a[data-drupal-selector="edit-header-share"]');

    $this->assertSession()->pageTextContains('First variation');
    $this->assertSession()->pageTextContains('Quantity: 1');
    $this->assertSession()->elementExists('css', 'a[data-drupal-selector="edit-items-1-details-edit"]');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-1"]');
    $this->assertSession()->elementExists('css', 'input[name="remove-1"]');

    $this->getSession()->getPage()->clickLink('Edit details');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertSession()->buttonExists('Update details');

    // Clear session, to verify access for anonymous user which is not owner.
    $this->getSession()->reset();

    $favorites = Favorites::load(1);
    $this->assertNotEmpty($favorites);
    $this->drupalGet($favorites->toUrl('user-form'));

    $this->assertSession()->elementExists('css', 'input[data-drupal-selector="edit-header-add-all-to-cart"]');
    $this->assertSession()->elementNotExists('css', 'a[data-drupal-selector="edit-header-share"]');
    $this->assertSession()->pageTextContains('First variation');
    $this->assertSession()->pageTextContains('Quantity: 1');
    $this->assertSession()->elementNotExists('css', 'a[data-drupal-selector="edit-items-1-details-edit"]');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-1"]');
    $this->assertSession()->elementNotExists('css', 'input[name="remove-1"]');
  }

}
