<?php

namespace Drupal\Tests\commerce_favorites\FunctionalJavascript;

use Drupal\Core\Url;
use Drupal\Tests\commerce\FunctionalJavascript\CommerceWebDriverTestBase;

/**
 * Tests the favorites user pages.
 *
 * @group commerce_favorites
 */
class FavoritesUserTest extends CommerceWebDriverTestBase {

  /**
   * The favorites.
   *
   * @var \Drupal\commerce_favorites\Entity\FavoritesInterface
   */
  protected $favorites;

  /**
   * A product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $variation1;

  /**
   * A product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $variation2;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'commerce_cart',
    'commerce_favorites',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->createEntity('commerce_product_variation_type', [
      'id' => 'test',
      'label' => 'Test',
      'orderItemType' => 'default',
      'generateTitle' => FALSE,
    ]);
    $entity_display = commerce_get_entity_display('commerce_product_variation', 'test', 'view');
    $entity_display->setComponent('title', [
      'label' => 'above',
      'type' => 'string',
    ]);
    $entity_display->save();

    $this->variation1 = $this->createEntity('commerce_product_variation', [
      'type' => 'test',
      'title' => 'First variation',
      'sku' => strtolower($this->randomMachineName()),
      'price' => [
        'number' => 9.99,
        'currency_code' => 'USD',
      ],
    ]);
    $this->variation2 = $this->createEntity('commerce_product_variation', [
      'type' => 'test',
      'title' => 'Second variation',
      'sku' => strtolower($this->randomMachineName()),
      'price' => [
        'number' => 20.99,
        'currency_code' => 'USD',
      ],
    ]);
    /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
    $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'My product',
      'variations' => [$this->variation1, $this->variation2],
      'stores' => [$this->store],
    ]);

    $this->favorites = $this->createEntity('commerce_favorites', [
      'type' => 'default',
      'title' => 'My favorites',
      'uid' => $this->adminUser->id(),
    ]);
  }

  /**
   * Tests the behavior of favorites pages when the favorites is empty.
   */
  public function testEmptyPage() {
    $this->drupalGet(Url::fromRoute('commerce_favorites.page'));
    $this->assertSession()->pageTextContains('Your favorites is empty');

    $this->drupalGet(Url::fromRoute('commerce_favorites.user_page', [
      'user' => $this->adminUser->id(),
    ]));
    $this->assertSession()->pageTextContains('Your favorites is empty');
  }

  /**
   * Tests the redirection to canonical/user_form pages.
   */
  public function testRedirects() {
    $favorites_item = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->variation1,
      'quantity' => 1,
    ]);
    $this->favorites->addItem($favorites_item);
    $this->favorites->save();
    $user_form_url = $this->favorites->toUrl('user-form')->setAbsolute();

    $this->drupalGet(Url::fromRoute('commerce_favorites.page'));
    $this->assertEquals($user_form_url->toString(), $this->getSession()->getCurrentUrl());

    $this->drupalGet(Url::fromRoute('commerce_favorites.user_page', [
      'user' => $this->adminUser->id(),
    ]));
    $this->assertEquals($user_form_url->toString(), $this->getSession()->getCurrentUrl());
  }

  /**
   * Tests the canonical page.
   */
  public function testCanonicalPage() {
    $favorites_item1 = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->variation1,
      'quantity' => 1,
    ]);
    $favorites_item2 = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->variation2,
      'quantity' => 2,
    ]);
    $this->favorites->setItems([$favorites_item1, $favorites_item2]);
    $this->favorites->save();

    $this->drupalGet($this->favorites->toUrl('canonical'));
    $this->assertSession()->elementExists('css', 'input[data-drupal-selector="edit-header-add-all-to-cart"]');
    $this->assertSession()->elementNotExists('css', 'a[data-drupal-selector="edit-header-share"]');

    $this->assertSession()->pageTextContains('First variation');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-1"]');
    $this->assertSession()->elementNotExists('css', 'input[name="remove-1"]');
    $this->assertSession()->pageTextContains('Second variation');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-2"]');
    $this->assertSession()->elementNotExists('css', 'input[name="remove-2"]');

    // Confirm that the "Add all to cart" button works.
    $cart_provider = $this->container->get('commerce_cart.cart_provider');
    $cart = $cart_provider->getCart('default');
    $this->assertEmpty($cart);
    $this->getSession()->getPage()->findButton('Add the entire list to cart')->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    $cart_provider->clearCaches();
    $cart = $cart_provider->getCart('default', $this->store);
    $this->assertCount(2, $cart->getItems());

    // Confirm that the "Add to cart" button works.
    $cart_manager = $this->container->get('commerce_cart.cart_manager');
    $cart_manager->emptyCart($cart);
    $button = $this->getSession()->getPage()->find('css', 'input[name="add-to-cart-2"]');
    $button->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
    $cart = $this->reloadEntity($cart);
    $this->assertCount(1, $cart->getItems());
    $order_items = $cart->getItems();
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = reset($order_items);
    $this->assertEquals($this->variation2->id(), $order_item->getPurchasedEntityId());
  }

  /**
   * Tests the user form.
   */
  public function testUserForm() {
    $favorites_item1 = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->variation1,
      'quantity' => 1,
    ]);
    $favorites_item2 = $this->createEntity('commerce_favorites_item', [
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->variation2,
      'quantity' => 2,
    ]);
    $this->favorites->setItems([$favorites_item1, $favorites_item2]);
    $this->favorites->save();

    $this->drupalGet($this->favorites->toUrl('user-form'));
    $this->assertSession()->elementExists('css', 'input[data-drupal-selector="edit-header-add-all-to-cart"]');
    $this->assertSession()->elementExists('css', 'a[data-drupal-selector="edit-header-share"]');

    $this->assertSession()->pageTextContains('First variation');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-1"]');
    $this->assertSession()->elementExists('css', 'input[name="remove-1"]');
    $this->assertSession()->pageTextContains('Second variation');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-2"]');
    $this->assertSession()->elementExists('css', 'input[name="remove-2"]');

    // Confirm that the "Add all to cart" button works.
    /** @var \Drupal\commerce_cart\CartProviderInterface $cart_provider */
    $cart_provider = $this->container->get('commerce_cart.cart_provider');
    $cart = $cart_provider->getCart('default');
    $this->assertEmpty($cart);
    $this->getSession()->getPage()->findButton('Add the entire list to cart')->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    $cart_provider->clearCaches();
    $cart = $cart_provider->getCart('default', $this->store);
    $this->assertCount(2, $cart->getItems());

    // Confirm that the "Add to cart" button works.
    $cart_manager = $this->container->get('commerce_cart.cart_manager');
    $cart_manager->emptyCart($cart);
    $button = $this->getSession()->getPage()->find('css', 'input[name="add-to-cart-2"]');
    $button->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
    $cart = $this->reloadEntity($cart);
    $this->assertCount(1, $cart->getItems());
    $order_items = $cart->getItems();
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = reset($order_items);
    $this->assertEquals($this->variation2->id(), $order_item->getPurchasedEntityId());

    // Confirm that the "Remove" button works.
    $button = $this->getSession()->getPage()->find('css', 'input[name="remove-2"]');
    $button->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertSession()->pageTextContains('Second variation has been removed from your favorites');
    $this->assertSession()->pageTextContains('First variation');
    $this->assertSession()->elementExists('css', 'input[name="add-to-cart-1"]');
    $this->assertSession()->elementExists('css', 'input[name="remove-1"]');
    $this->assertSession()->elementNotExists('css', 'input[name="add-to-cart-2"]');
    $this->assertSession()->elementNotExists('css', 'input[name="remove-2"]');
    $this->favorites = $this->reloadEntity($this->favorites);
    $this->assertFalse($this->favorites->hasItem($favorites_item2));
  }

}
