<?php

namespace Drupal\Tests\commerce_favorites\Kernel\Entity;

use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\FavoritesPurchase;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\Tests\commerce_cart\Kernel\CartKernelTestBase;

/**
 * Tests the favorites item entity.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\Entity\FavoritesItem
 * @group commerce_favorites
 */
class FavoritesItemTest extends CartKernelTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'commerce_favorites',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('commerce_favorites');
    $this->installEntitySchema('commerce_favorites_item');
    $this->installConfig(['commerce_favorites']);
  }

  /**
   * Tests the favorites item entity and its methods.
   *
   * @covers ::getFavoritesId
   * @covers ::getFavorites
   * @covers ::getPurchasableEntity
   * @covers ::getPurchasableEntityId
   * @covers ::getTitle
   * @covers ::getQuantity
   * @covers ::setQuantity
   * @covers ::getComment
   * @covers ::setComment
   * @covers ::getPriority
   * @covers ::setPriority
   * @covers ::getPurchases
   * @covers ::setPurchases
   * @covers ::addPurchase
   * @covers ::removePurchase
   * @covers ::getPurchasedQuantity
   * @covers ::getLastPurchasedTime
   * @covers ::getCreatedTime
   * @covers ::getChangedTime
   */
  public function testFavoritesItem() {
    /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
    $product = Product::create([
      'type' => 'default',
      'title' => 'My Product Title',
    ]);
    $product->save();
    /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $variation */
    $variation = ProductVariation::create([
      'type' => 'default',
      'product_id' => $product->id(),
    ]);
    $variation->save();
    $variation = $this->reloadEntity($variation);
    /** @var \Drupal\commerce_favorites\FavoritesItemStorageInterface $favorites_item_storage */
    $favorites_item_storage = $this->container->get('entity_type.manager')->getStorage('commerce_favorites_item');

    $favorites = Favorites::create([
      'type' => 'default',
      'state' => 'completed',
    ]);
    $favorites_item = $favorites_item_storage->createFromPurchasableEntity($variation, [
      'quantity' => 2,
    ]);
    $favorites_item->save();
    $favorites->setItems([$favorites_item]);
    $favorites->save();
    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
    $favorites_item = $this->reloadEntity($favorites_item);

    $this->assertEquals($favorites_item->getFavoritesId(), $favorites->id());
    $this->assertEquals($favorites_item->getFavorites()->id(), $favorites->id());
    $this->assertEquals($favorites_item->getPurchasableEntity()->id(), $variation->id());
    $this->assertEquals($favorites_item->getPurchasableEntityId(), $variation->id());
    $this->assertEquals($favorites_item->getTitle(), $variation->label());
    $this->assertEquals($favorites_item->getQuantity(), 2);
    $favorites_item->setQuantity(3);
    $this->assertEquals($favorites_item->getQuantity(), 3);

    $favorites_item->setComment('My comment');
    $this->assertEquals($favorites_item->getComment(), 'My comment');

    $favorites_item->setPriority(100);
    $this->assertEquals($favorites_item->getPriority(), 100);

    $time = 635879700;
    $this->assertNull($favorites_item->getLastPurchasedTime());
    $purchase = new FavoritesPurchase(10, 2, $time + 10);
    $favorites_item->setPurchases([$purchase]);
    $purchases = $favorites_item->getPurchases();
    $this->assertEquals([$purchase], $purchases);
    $this->assertEquals(2, $favorites_item->getPurchasedQuantity());
    $this->assertEquals($time + 10, $favorites_item->getLastPurchasedTime());

    $another_purchase = new FavoritesPurchase(11, 3, $time + 5);
    $favorites_item->addPurchase($another_purchase);
    $purchases = $favorites_item->getPurchases();
    $this->assertEquals([$purchase, $another_purchase], $purchases);
    $this->assertEquals(5, $favorites_item->getPurchasedQuantity());
    $this->assertEquals($time + 10, $favorites_item->getLastPurchasedTime());

    $favorites_item->removePurchase($purchase);
    $purchases = $favorites_item->getPurchases();
    $this->assertEquals([$another_purchase], $purchases);
    $this->assertEquals(3, $favorites_item->getPurchasedQuantity());
    $this->assertEquals($time + 5, $favorites_item->getLastPurchasedTime());

    $favorites_item->setCreatedTime($time);
    $this->assertEquals($favorites_item->getCreatedTime(), $time);
  }

}
