<?php

namespace Drupal\Tests\commerce_favorites\Kernel\Entity;

use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\Entity\FavoritesItem;
use Drupal\commerce_favorites\Entity\FavoritesType;
use Drupal\profile\Entity\Profile;
use Drupal\Tests\commerce_favorites\Kernel\FavoritesKernelTestBase;

/**
 * Tests the favorites entity.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\Entity\Favorites
 * @group commerce_favorites
 */
class FavoritesTest extends FavoritesKernelTestBase {

  /**
   * A sample user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp():void {
    parent::setUp();

    $user = $this->createUser();
    $this->user = $this->reloadEntity($user);
  }

  /**
   * Tests the favorites entity and its methods.
   *
   * @covers ::getCode
   * @covers ::setCode
   * @covers ::getName
   * @covers ::setName
   * @covers ::getOwner
   * @covers ::setOwner
   * @covers ::getOwnerId
   * @covers ::setOwnerId
   * @covers ::getShippingProfile
   * @covers ::setShippingProfile
   * @covers ::getItems
   * @covers ::setItems
   * @covers ::hasItems
   * @covers ::addItem
   * @covers ::removeItem
   * @covers ::hasItem
   * @covers ::getCreatedTime
   * @covers ::setCreatedTime
   * @covers ::isPublic
   * @covers ::setPublic
   * @covers ::getKeepPurchasedItems
   * @covers ::setKeepPurchasedItems
   */
  public function testFavorites() {
    $profile = Profile::create([
      'type' => 'customer',
    ]);
    $profile->save();
    $profile = $this->reloadEntity($profile);

    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
    $favorites_item = FavoritesItem::create([
      'type' => 'commerce_product_variation',
    ]);
    $favorites_item->save();
    $favorites_item = $this->reloadEntity($favorites_item);
    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $another_favorites_item */
    $another_favorites_item = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'quantity' => '2',
    ]);
    $another_favorites_item->save();
    $another_favorites_item = $this->reloadEntity($another_favorites_item);

    $favorites = Favorites::create([
      'type' => 'default',
    ]);
    $favorites->save();

    $this->assertNotEmpty($favorites->getCode());
    $favorites->setCode('new_code');
    $this->assertEquals('new_code', $favorites->getCode());
    $this->assertEquals('/favorites/new_code', $favorites->toUrl()->toString());

    $favorites->setName('My favorites');
    $this->assertEquals('My favorites', $favorites->getName());

    $favorites->setOwner($this->user);
    $this->assertEquals($this->user, $favorites->getOwner());
    $this->assertEquals($this->user->id(), $favorites->getOwnerId());
    $favorites->setOwnerId(0);
    $this->assertEquals(NULL, $favorites->getOwner());
    $favorites->setOwnerId($this->user->id());
    $this->assertEquals($this->user, $favorites->getOwner());
    $this->assertEquals($this->user->id(), $favorites->getOwnerId());

    $favorites->setShippingProfile($profile);
    $this->assertEquals($profile, $favorites->getShippingProfile());

    $favorites->setItems([$favorites_item, $another_favorites_item]);
    $this->assertEquals([$favorites_item, $another_favorites_item], $favorites->getItems());
    $this->assertTrue($favorites->hasItems());
    $favorites->removeItem($another_favorites_item);
    $this->assertEquals([$favorites_item], $favorites->getItems());
    $this->assertTrue($favorites->hasItem($favorites_item));
    $this->assertFalse($favorites->hasItem($another_favorites_item));
    $favorites->addItem($another_favorites_item);
    $this->assertEquals([$favorites_item, $another_favorites_item], $favorites->getItems());
    $this->assertTrue($favorites->hasItem($another_favorites_item));

    $favorites->setCreatedTime(635879700);
    $this->assertEquals(635879700, $favorites->getCreatedTime());

    $this->assertFalse($favorites->isPublic());
    $favorites->setPublic(TRUE);
    $this->assertTrue($favorites->isPublic());

    $this->assertTrue($favorites->getKeepPurchasedItems());
    $favorites->setKeepPurchasedItems(FALSE);
    $this->assertFalse($favorites->getKeepPurchasedItems());
  }

  /**
   * Tests the default favorites functionality.
   */
  public function testDefaultFavorites() {
    $favorites_type = FavoritesType::create([
      'id' => 'test_defaults',
      'label' => 'test_defaults',
    ]);
    $favorites_type->save();

    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites1 */
    $favorites1 = Favorites::create([
      'type' => $favorites_type->id(),
      'uid' => $this->user->id(),
    ]);
    $favorites1->save();
    // Confirm that the favorites was set as default.
    $this->assertTrue($favorites1->isDefault());

    /** @var \Drupal\commerce_favorites\Entity\FavoritesInterface $favorites2 */
    $favorites2 = Favorites::create([
      'type' => $favorites_type->id(),
      'uid' => $this->user->id(),
    ]);
    $favorites2->setDefault(TRUE);
    $favorites2->save();

    // Confirm that setting the second favorites as default removed the
    // flag from the first favorites.
    $favorites2 = $this->reloadEntity($favorites2);
    $favorites1 = $this->reloadEntity($favorites1);
    $this->assertTrue($favorites2->isDefault());
    $this->assertFalse($favorites1->isDefault());
  }

}
