<?php

namespace Drupal\Tests\commerce_favorites\Kernel;

use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_product\Entity\ProductVariationType;
use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\commerce_favorites\Entity\FavoritesItem;
use Drupal\commerce_favorites\Entity\FavoritesItemInterface;
use Drupal\commerce_favorites\Entity\FavoritesType;

/**
 * Tests the favorites assignment.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\FavoritesAssignment
 * @group commerce_favorites
 */
class FavoritesAssignmentTest extends FavoritesKernelTestBase {

  /**
   * Anonymous user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $anonymousUser;

  /**
   * Registered user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $authenticatedUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The favorites assignment.
   *
   * @var \Drupal\commerce_favorites\FavoritesAssignmentInterface
   */
  protected $favoritesAssignment;

  /**
   * First product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $productVariationFirst;

  /**
   * Second product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $productVariationSecond;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Turn off title generation to allow explicit values to be used.
    $variation_type = ProductVariationType::load('default');
    $variation_type->setGenerateTitle(FALSE);
    $variation_type->save();

    $favorites_type = FavoritesType::create([
      'id' => 'test_1',
      'label' => 'Test 1',
    ]);
    $favorites_type->save();

    $another_favorites_type = FavoritesType::create([
      'id' => 'test_2',
      'label' => 'Test 2',
    ]);
    $another_favorites_type->save();

    $this->productVariationFirst = ProductVariation::create([
      'type' => 'default',
      'sku' => 'product_1',
      'title' => 'Product 1',
      'status' => 1,
    ]);
    $this->productVariationFirst->save();

    $this->productVariationSecond = ProductVariation::create([
      'type' => 'default',
      'sku' => 'product_2',
      'title' => 'Product 2',
      'status' => 1,
    ]);
    $this->productVariationSecond->save();

    $this->anonymousUser = $this->createUser([
      'uid' => 0,
      'name' => '',
      'status' => 0,
    ]);
    $this->authenticatedUser = $this->createUser();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->favoritesAssignment = $this->container->get('commerce_favorites.favorites_assignment');
    $this->config('commerce_favorites.settings')->set('allow_multiple', 1)->save();
  }

  /**
   * Tests favorites assignment.
   *
   * @covers ::assign
   */
  public function testAssignFavorites() {
    $default_favorites = $this->entityTypeManager->getStorage('commerce_favorites')->loadDefaultByUser($this->authenticatedUser, 'test_1');
    $this->assertEmpty($default_favorites);

    $favorites_item = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationFirst,
    ]);
    $favorites_item->save();
    $this->assertInstanceOf(FavoritesItemInterface::class, $favorites_item);

    $favorites = Favorites::create([
      'type' => 'test_1',
      'name' => 'First favorites',
      'favorites_items' => [$favorites_item],
      'uid' => 0,
    ]);
    $favorites->save();
    $this->assertInstanceOf(FavoritesInterface::class, $favorites);

    $this->favoritesAssignment->assign($favorites, $this->authenticatedUser);
    $this->reloadEntity($favorites);

    $this->assertEqual($favorites->getOwnerId(), $this->authenticatedUser->id());

  }

  /**
   * Tests multiple favorites assignment.
   *
   * @covers ::assignMultiple
   */
  public function testAssignMultipleFavorites() {
    $default_favorites = $this->entityTypeManager->getStorage('commerce_favorites')->loadDefaultByUser($this->authenticatedUser, 'test_1');
    $this->assertEmpty($default_favorites);

    $favorites_item_1 = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationFirst,
    ]);
    $favorites_item_1->save();
    $this->assertInstanceOf(FavoritesItemInterface::class, $favorites_item_1);

    $favorites_item_2 = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationSecond,
    ]);
    $favorites_item_2->save();
    $this->assertInstanceOf(FavoritesItemInterface::class, $favorites_item_1);

    $favorites_1 = Favorites::create([
      'type' => 'test_1',
      'name' => 'First favorites',
      'favorites_items' => [$favorites_item_1],
      'uid' => 0,
    ]);
    $favorites_1->save();
    $this->assertInstanceOf(FavoritesInterface::class, $favorites_1);

    $favorites_2 = Favorites::create([
      'type' => 'test_1',
      'name' => 'First favorites',
      'favorites_items' => [$favorites_item_2, $favorites_item_1],
      'uid' => 0,
    ]);
    $favorites_2->save();
    $this->assertInstanceOf(FavoritesInterface::class, $favorites_2);

    $favoritess = [
      $favorites_1,
      $favorites_2,
    ];

    $this->favoritesAssignment->assignMultiple($favoritess, $this->authenticatedUser);
    $this->reloadEntity($favorites_1);
    $this->reloadEntity($favorites_2);

    $this->assertEqual($favorites_1->getOwnerId(), $this->authenticatedUser->id());
    $this->assertEqual($favorites_2->getOwnerId(), $this->authenticatedUser->id());

    $this->config('commerce_favorites.settings')->set('allow_multiple', 0)->save();
    $another_user = $this->createUser();
    $default_favorites = $this->entityTypeManager->getStorage('commerce_favorites')->loadDefaultByUser($another_user, 'test_1');
    $this->assertEmpty($default_favorites);

    $favorites = Favorites::create([
      'type' => 'test_1',
      'name' => 'First favorites',
      'favorites_items' => [$favorites_item_1],
      'uid' => $another_user->id(),
      'is_default' => TRUE,
    ]);
    $favorites->save();
    $default_favorites = $this->entityTypeManager->getStorage('commerce_favorites')->loadDefaultByUser($another_user, 'test_1');
    $this->assertNotEmpty($default_favorites);

    $anonymous_favorites = Favorites::create([
      'type' => 'test_1',
      'name' => 'Anonymous favorites',
      'favorites_items' => [$favorites_item_2],
      'uid' => 0,
    ]);
    $anonymous_favorites->save();
    $this->favoritesAssignment->assignMultiple([$anonymous_favorites], $another_user);
    $favorites = $this->reloadEntity($favorites);
    $this->assertCount(2, $favorites->getItems());

    $anonymous_favorites = $this->reloadEntity($anonymous_favorites);
    $this->assertNull($anonymous_favorites);

    $default_favorites = $this->entityTypeManager->getStorage('commerce_favorites')->loadDefaultByUser($another_user, 'test_2');
    $this->assertEmpty($default_favorites);
    $anonymous_favorites = Favorites::create([
      'type' => 'test_2',
      'name' => 'Anonymous favorites',
      'favorites_items' => [$favorites_item_2],
      'uid' => 0,
    ]);
    $anonymous_favorites->save();
    $this->favoritesAssignment->assignMultiple([$anonymous_favorites], $another_user);
    $anonymous_favorites = $this->reloadEntity($anonymous_favorites);
    $this->assertEqual($anonymous_favorites->getOwnerId(), $another_user->id());
  }

}
