<?php

namespace Drupal\Tests\commerce_favorites\Kernel;

use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\Entity\FavoritesItem;

/**
 * Tests the favorites item access control.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\FavoritesItemAccessControlHandler
 * @group commerce_favorites
 */
class FavoritesItemAccessTest extends FavoritesKernelTestBase {

  /**
   * A sample user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create uid: 1 here so that it's skipped in test cases.
    $admin_user = $this->createUser();
    $regular_user = $this->createUser(['uid' => 2]);
    \Drupal::currentUser()->setAccount($regular_user);
  }

  /**
   * @covers ::checkAccess
   */
  public function testAccess() {
    /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $variation */
    $variation = ProductVariation::create([
      'type' => 'default',
      'sku' => $this->randomMachineName(),
      'title' => $this->randomString(),
      'status' => 1,
    ]);
    $variation->save();

    /** @var \Drupal\commerce_favorites\Entity\FavoritesItemInterface $favorites_item */
    $favorites_item = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $variation,
    ]);
    $favorites_item->save();

    $favorites = Favorites::create([
      'type' => 'default',
      'name' => 'My favorites',
      'favorites_items' => [$favorites_item],
    ]);
    $favorites->save();
    $favorites_item = $this->reloadEntity($favorites_item);

    $account = $this->createUser([], ['access administration pages']);
    $this->assertFalse($favorites_item->access('view', $account));
    $this->assertFalse($favorites_item->access('update', $account));
    $this->assertFalse($favorites_item->access('delete', $account));

    $account = $this->createUser([], ['view any commerce_favorites']);
    $this->assertTrue($favorites_item->access('view', $account));
    $this->assertFalse($favorites_item->access('update', $account));
    $this->assertFalse($favorites_item->access('delete', $account));

    $account = $this->createUser([], ['update any default commerce_favorites']);
    $this->assertFalse($favorites_item->access('view', $account));
    $this->assertFalse($favorites_item->access('update', $account));
    $this->assertFalse($favorites_item->access('delete', $account));

    $account = $this->createUser([], [
      'manage commerce_product_variation commerce_favorites_item',
    ]);
    $this->assertFalse($favorites_item->access('view', $account));
    $this->assertTrue($favorites_item->access('update', $account));
    $this->assertTrue($favorites_item->access('delete', $account));

    $account = $this->createUser([], ['administer commerce_favorites']);
    $this->assertTrue($favorites_item->access('view', $account));
    $this->assertTrue($favorites_item->access('update', $account));
    $this->assertTrue($favorites_item->access('delete', $account));

    // Broken favorites reference.
    $favorites_item->set('favorites_id', '999');
    $account = $this->createUser([], ['manage commerce_product_variation commerce_favorites_item']);
    $this->assertFalse($favorites_item->access('view', $account));
    $this->assertFalse($favorites_item->access('update', $account));
    $this->assertFalse($favorites_item->access('delete', $account));

    $account = $this->createUser([], ['view own commerce_favorites']);
    $favorites->setOwnerId($account->id());
    $favorites->save();
    $favorites_item = $this->reloadEntity($favorites_item);
    $this->assertTrue($favorites_item->access('view', $account));
    $this->assertFalse($favorites_item->access('update', $account));
    $this->assertFalse($favorites_item->access('delete', $account));
  }

  /**
   * @covers ::checkCreateAccess
   */
  public function testCreateAccess() {
    $access_control_handler = \Drupal::entityTypeManager()->getAccessControlHandler('commerce_favorites_item');

    $account = $this->createUser([], ['access content']);
    $this->assertFalse($access_control_handler->createAccess('test', $account));

    $account = $this->createUser([], ['administer commerce_favorites']);
    $this->assertTrue($access_control_handler->createAccess('default', $account));
    $this->assertTrue($access_control_handler->createAccess('commerce_product_variation', $account));

    $account = $this->createUser([], ['manage commerce_product_variation commerce_favorites_item']);
    $this->assertTrue($access_control_handler->createAccess('commerce_product_variation', $account));
  }

}
