<?php

namespace Drupal\Tests\commerce_favorites\Kernel;

use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_product\Entity\ProductVariationType;
use Drupal\commerce_favorites\Entity\Favorites;
use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\commerce_favorites\Entity\FavoritesItem;
use Drupal\commerce_favorites\Entity\FavoritesItemInterface;
use Drupal\commerce_favorites\Entity\FavoritesType;

/**
 * Tests the favorites manager.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\FavoritesManager
 * @group commerce_favorites
 */
class FavoritesManagerTest extends FavoritesKernelTestBase {

  /**
   * Anonymous user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $anonymousUser;

  /**
   * Registered user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $authenticatedUser;

  /**
   * The purchasable entity.
   *
   * @var \Drupal\commerce\PurchasableEntityInterface
   */
  protected $purchasableEntity;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The favorites manager.
   *
   * @var \Drupal\commerce_favorites\FavoritesManagerInterface
   */
  protected $favoritesManager;

  /**
   * First product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $productVariationFirst;

  /**
   * Second product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $productVariationSecond;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Turn off title generation to allow explicit values to be used.
    $variation_type = ProductVariationType::load('default');
    $variation_type->setGenerateTitle(FALSE);
    $variation_type->save();

    $favorites_type = FavoritesType::create([
      'id' => 'test',
      'label' => 'Test',
    ]);
    $favorites_type->save();

    $this->anonymousUser = $this->createUser([
      'uid' => 0,
      'name' => '',
      'status' => 0,
    ]);
    $this->authenticatedUser = $this->createUser();

    $this->productVariationFirst = ProductVariation::create([
      'type' => 'default',
      'sku' => 'product_1',
      'title' => 'Product 1',
      'status' => 1,
    ]);
    $this->productVariationFirst->save();

    $this->productVariationSecond = ProductVariation::create([
      'type' => 'default',
      'sku' => 'product_2',
      'title' => 'Product 2',
      'status' => 1,
    ]);
    $this->productVariationSecond->save();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->favoritesManager = $this->container->get('commerce_favorites.favorites_manager');
  }

  /**
   * Tests adding a purchasable entity to a favorites.
   *
   * @covers ::addEntity
   */
  public function testAddEntity() {
    $favorites = Favorites::create([
      'type' => 'test',
      'name' => 'My favorites',
    ]);
    $favorites->save();

    $favorites_item = $this->favoritesManager->addEntity($favorites, $this->productVariationFirst, 3);
    $this->assertInstanceOf(FavoritesItemInterface::class, $favorites_item);
    $this->assertEquals(3, $favorites_item->getQuantity());
    $this->assertEquals('Product 1', $favorites_item->getTitle());
    $this->assertTrue($favorites->hasItem($favorites_item));
  }

  /**
   * Tests emptying a favorites.
   *
   * @covers ::emptyFavorites
   */
  public function testEmptyFavorites() {
    $favorites = Favorites::create([
      'type' => 'test',
      'name' => 'My favorites',
    ]);
    $favorites->save();
    $this->favoritesManager->addEntity($favorites, $this->productVariationSecond);

    $this->assertTrue($favorites->hasItems());
    $this->favoritesManager->emptyFavorites($favorites);
    $this->assertFalse($favorites->hasItems());
  }

  /**
   * Tests favorites merge.
   *
   * @covers ::merge
   */
  public function testMergeFavorites() {
    $favorites_item_1 = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationFirst,
    ]);

    $source = Favorites::create([
      'type' => 'test_1',
      'name' => 'Source favorites',
      'favorites_items' => [$favorites_item_1],
      'uid' => 1,
    ]);
    $source->save();
    $this->reloadEntity($source);
    $this->assertEqual($source->id(), $this->authenticatedUser->id());

    $favorites_item_2 = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationFirst,
    ]);
    $favorites_item_2->save();
    $this->assertInstanceOf(FavoritesItemInterface::class, $favorites_item_2);

    $target = Favorites::create([
      'type' => 'test_1',
      'name' => 'Target favorites',
      'favorites_items' => [$favorites_item_2],
      'uid' => 1,
    ]);
    $target->save();
    $this->assertEqual(count($target->getItems()), 1);
    $this->assertInstanceOf(FavoritesInterface::class, $target);

    $this->favoritesManager->merge($source, $target);
    $this->reloadEntity($source);
    $this->reloadEntity($target);
    $source = $this->entityTypeManager->getStorage('commerce_favorites')->load($source->id());
    $target = $this->entityTypeManager->getStorage('commerce_favorites')->load($target->id());
    $this->assertInstanceOf(FavoritesInterface::class, $target);
    $this->assertNotInstanceOf(FavoritesInterface::class, $source);

    $this->assertEqual(count($target->getItems()), 2);
  }

  /**
   * Tests removing a favorites item from a favorites.
   *
   * @covers ::removeFavoritesItem
   */
  public function testRemoveFavoritesItem() {
    $favorites_item = FavoritesItem::create([
      'type' => 'commerce_product_variation',
      'purchasable_entity' => $this->productVariationFirst,
    ]);
    $favorites_item->save();
    $favorites = Favorites::create([
      'type' => 'test',
      'favorites_items' => [$favorites_item],
      'name' => 'My favorites',
    ]);
    $favorites->save();
    $this->assertTrue($favorites->hasItems());

    $this->favoritesManager->removeFavoritesItem($favorites, $favorites_item);
    $this->assertFalse($favorites->hasItems());
  }

}
