<?php

namespace Drupal\Tests\commerce_favorites\Kernel;

use Drupal\commerce_favorites\Entity\FavoritesInterface;
use Drupal\commerce_favorites\Exception\DuplicateFavoritesException;

/**
 * Tests the favorites provider.
 *
 * @coversDefaultClass \Drupal\commerce_favorites\FavoritesProvider
 * @group commerce_favorites
 */
class FavoritesProviderTest extends FavoritesKernelTestBase {

  /**
   * Anonymous user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $anonymousUser;

  /**
   * Registered user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $authenticatedUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The favorites provider.
   *
   * @var \Drupal\commerce_favorites\FavoritesProviderInterface
   */
  protected $favoritesProvider;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->anonymousUser = $this->createUser([
      'uid' => 0,
      'name' => '',
      'status' => 0,
    ]);
    $this->authenticatedUser = $this->createUser();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->favoritesProvider = $this->container->get('commerce_favorites.favorites_provider');
  }

  /**
   * Tests favorites creation for an anonymous user.
   *
   * @covers ::createFavorites
   */
  public function testCreateAnonymousFavorites() {
    $favorites_type = 'default';
    $favorites = $this->favoritesProvider->createFavorites($favorites_type, $this->anonymousUser);
    $this->assertInstanceOf(FavoritesInterface::class, $favorites);

    // Trying to recreate the same favorites should throw an exception.
    $this->expectException(DuplicateFavoritesException::class);
    $this->favoritesProvider->createFavorites($favorites_type, $this->anonymousUser);
  }

  /**
   * Tests getting an anonymous user's favorites.
   *
   * @covers ::getFavorites
   * @covers ::getFavoritesId
   * @covers ::getFavoritess
   * @covers ::getFavoritesIds
   */
  public function testGetAnonymousFavorites() {
    $this->favoritesProvider->createFavorites('default', $this->anonymousUser);
    $favorites = $this->favoritesProvider->getFavorites('default', $this->anonymousUser);
    $this->assertInstanceOf(FavoritesInterface::class, $favorites);

    $favorites_id = $this->favoritesProvider->getFavoritesId('default', $this->anonymousUser);
    $this->assertEquals(1, $favorites_id);

    $favoritess = $this->favoritesProvider->getFavoritess($this->anonymousUser);
    $this->assertContainsOnlyInstancesOf(FavoritesInterface::class, $favoritess);

    $favorites_ids = $this->favoritesProvider->getFavoritesIds($this->anonymousUser);
    $this->assertContains(1, $favorites_ids);
  }

  /**
   * Tests creating a favorites for an authenticated user.
   *
   * @covers ::createFavorites
   */
  public function testCreateAuthenticatedFavorites() {
    $favorites = $this->favoritesProvider->createFavorites('default', $this->authenticatedUser);
    $this->assertInstanceOf(FavoritesInterface::class, $favorites);

    // Trying to recreate the same favorites should throw an exception.
    $this->expectException(DuplicateFavoritesException::class);
    $this->favoritesProvider->createFavorites('default', $this->authenticatedUser);
  }

  /**
   * Tests getting an authenticated user's favorites.
   *
   * @covers ::getFavorites
   * @covers ::getFavoritesId
   * @covers ::getFavoritess
   * @covers ::getFavoritesIds
   */
  public function testGetAuthenticatedFavorites() {
    $this->favoritesProvider->createFavorites('default', $this->authenticatedUser);

    $favorites = $this->favoritesProvider->getFavorites('default', $this->authenticatedUser);
    $this->assertInstanceOf(FavoritesInterface::class, $favorites);

    $favorites_id = $this->favoritesProvider->getFavoritesId('default', $this->authenticatedUser);
    $this->assertEquals(1, $favorites_id);

    $favoritess = $this->favoritesProvider->getFavoritess($this->authenticatedUser);
    $this->assertContainsOnlyInstancesOf(FavoritesInterface::class, $favoritess);

    $favorites_ids = $this->favoritesProvider->getFavoritesIds($this->authenticatedUser);
    $this->assertContains(1, $favorites_ids);
  }

}
